@@ -31,6 +31,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 
 #include <libedataserver/e-url.h>
 #include <libedataserver/e-flag.h>
@@ -49,9 +50,15 @@
 
 #include <libxml/parser.h>
 #include <libxml/xmlreader.h>
+#include <libxml/xpath.h>
+#include <libxml/xpathInternals.h>
+
+#define EDB_ERROR(_code) e_data_book_create_error (E_DATA_BOOK_STATUS_ ## _code, NULL)
+#define EDB_ERROR_EX(_code, _msg) e_data_book_create_error (E_DATA_BOOK_STATUS_ ## _code, _msg)
 
 #define USERAGENT             "Evolution/" VERSION
 #define WEBDAV_CLOSURE_NAME   "EBookBackendWebdav.BookView::closure"
+#define WEBDAV_CTAG_KEY "WEBDAV_CTAG"
 
 G_DEFINE_TYPE (EBookBackendWebdav, e_book_backend_webdav, E_TYPE_BOOK_BACKEND)
 
@@ -65,6 +72,7 @@ struct _EBookBackendWebdavPrivate {
 	gchar             *uri;
 	gchar              *username;
 	gchar              *password;
+	gboolean supports_getctag;
 
 	EBookBackendCache *cache;
 };
@@ -137,8 +145,8 @@ download_contact(EBookBackendWebdav *webdav, const gchar *uri)
 	guint        status;
 
 	message = soup_message_new(SOUP_METHOD_GET, uri);
-	soup_message_headers_append(message->request_headers,
-				    "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "Connection", "close");
 
 	status = soup_session_send_message(webdav->priv->session, message);
 	if (status != 200) {
@@ -198,8 +206,8 @@ upload_contact(EBookBackendWebdav *webdav, EContact *contact)
 	}
 
 	message = soup_message_new(SOUP_METHOD_PUT, uri);
-	soup_message_headers_append(message->request_headers,
-				    "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "Connection", "close");
 
 	property = e_source_get_property(source, "avoid_ifmatch");
 	if (property != NULL && strcmp(property, "1") == 0) {
@@ -230,7 +238,7 @@ upload_contact(EBookBackendWebdav *webdav, EContact *contact)
 	e_contact_set(contact, E_CONTACT_UID, NULL);
 	e_contact_set(contact, E_CONTACT_REV, NULL);
 	request = e_vcard_to_string(E_VCARD(contact), EVC_FORMAT_VCARD_30);
-	soup_message_set_request(message, "text/x-vcard", SOUP_MEMORY_TEMPORARY,
+	soup_message_set_request(message, "text/vcard", SOUP_MEMORY_TEMPORARY,
 				 request, strlen(request));
 
 	status   = soup_session_send_message(webdav->priv->session, message);
@@ -267,8 +275,8 @@ upload_contact(EBookBackendWebdav *webdav, EContact *contact)
 	return status;
 }
 
-static GNOME_Evolution_Addressbook_CallStatus
-e_book_backend_handle_auth_request(EBookBackendWebdav *webdav)
+static GError *
+webdav_handle_auth_request(EBookBackendWebdav *webdav)
 {
 	EBookBackendWebdavPrivate *priv = webdav->priv;
 
@@ -278,9 +286,9 @@ e_book_backend_handle_auth_request(EBookBackendWebdav *webdav)
 		g_free(priv->password);
 		priv->password = NULL;
 
-		return GNOME_Evolution_Addressbook_AuthenticationFailed;
+		return EDB_ERROR (AUTHENTICATION_FAILED);
 	} else {
-		return GNOME_Evolution_Addressbook_AuthenticationRequired;
+		return EDB_ERROR (AUTHENTICATION_REQUIRED);
 	}
 }
 
@@ -294,9 +302,8 @@ e_book_backend_webdav_create_contact(EBookBackend *backend,
 	gchar                     *uid;
 	guint                      status;
 
-	if (priv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
-		e_data_book_respond_create(book, opid,
-				GNOME_Evolution_Addressbook_RepositoryOffline, NULL);
+	if (priv->mode == E_DATA_BOOK_MODE_LOCAL) {
+		e_data_book_respond_create (book, opid, EDB_ERROR (REPOSITORY_OFFLINE), NULL);
 		return;
 	}
 
@@ -315,14 +322,12 @@ e_book_backend_webdav_create_contact(EBookBackend *backend,
 	if (status != 201 && status != 204) {
 		g_object_unref(contact);
 		if (status == 401 || status == 407) {
-			GNOME_Evolution_Addressbook_CallStatus res
-				= e_book_backend_handle_auth_request(webdav);
-			e_data_book_respond_create(book, opid, res, NULL);
+			e_data_book_respond_create (book, opid, webdav_handle_auth_request (webdav), NULL);
 		} else {
-			g_warning("create resource '%s' failed with http status: %d",
-				  uid, status);
-			e_data_book_respond_create(book, opid,
-					GNOME_Evolution_Addressbook_OtherError, NULL);
+			e_data_book_respond_create (book, opid,
+					e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR,
+						_("Create resource '%s' failed with http status: %d"), uid, status),
+					NULL);
 		}
 		g_free(uid);
 		return;
@@ -338,8 +343,8 @@ e_book_backend_webdav_create_contact(EBookBackend *backend,
 		g_object_unref(contact);
 
 		if (new_contact == NULL) {
-			e_data_book_respond_create(book, opid,
-					GNOME_Evolution_Addressbook_OtherError, NULL);
+			e_data_book_respond_create (book, opid,
+					EDB_ERROR (OTHER_ERROR), NULL);
 			g_free(uid);
 			return;
 		}
@@ -347,8 +352,7 @@ e_book_backend_webdav_create_contact(EBookBackend *backend,
 	}
 
 	e_book_backend_cache_add_contact(priv->cache, contact);
-	e_data_book_respond_create(book, opid,
-			GNOME_Evolution_Addressbook_Success, contact);
+	e_data_book_respond_create (book, opid, EDB_ERROR (SUCCESS), contact);
 
 	if (contact)
 		g_object_unref(contact);
@@ -362,8 +366,8 @@ delete_contact(EBookBackendWebdav *webdav, const gchar *uri)
 	guint        status;
 
 	message = soup_message_new(SOUP_METHOD_DELETE, uri);
-	soup_message_headers_append(message->request_headers,
-				    "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "Connection", "close");
 
 	status = soup_session_send_message(webdav->priv->session, message);
 	g_object_unref(message);
@@ -380,9 +384,9 @@ e_book_backend_webdav_remove_contacts(EBookBackend *backend,
 	GList                     *deleted_ids = NULL;
 	GList                     *list;
 
-	if (priv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
-		e_data_book_respond_create(book, opid,
-				GNOME_Evolution_Addressbook_RepositoryOffline, NULL);
+	if (priv->mode == E_DATA_BOOK_MODE_LOCAL) {
+		e_data_book_respond_remove_contacts (book, opid,
+				EDB_ERROR (REPOSITORY_OFFLINE), NULL);
 		return;
 	}
 
@@ -393,9 +397,7 @@ e_book_backend_webdav_remove_contacts(EBookBackend *backend,
 		status = delete_contact(webdav, uid);
 		if (status != 204) {
 			if (status == 401 || status == 407) {
-				GNOME_Evolution_Addressbook_CallStatus res
-					= e_book_backend_handle_auth_request(webdav);
-				e_data_book_respond_remove_contacts(book, opid, res,
+				e_data_book_respond_remove_contacts (book, opid, webdav_handle_auth_request (webdav),
 								    deleted_ids);
 			} else {
 				g_warning("DELETE failed with http status %d", status);
@@ -407,7 +409,7 @@ e_book_backend_webdav_remove_contacts(EBookBackend *backend,
 	}
 
 	e_data_book_respond_remove_contacts (book, opid,
-			GNOME_Evolution_Addressbook_Success,  deleted_ids);
+			EDB_ERROR (SUCCESS),  deleted_ids);
 }
 
 static void
@@ -421,9 +423,9 @@ e_book_backend_webdav_modify_contact(EBookBackend *backend,
 	const gchar                *etag;
 	guint status;
 
-	if (priv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
+	if (priv->mode == E_DATA_BOOK_MODE_LOCAL) {
 		e_data_book_respond_create(book, opid,
-				GNOME_Evolution_Addressbook_RepositoryOffline, NULL);
+				EDB_ERROR (REPOSITORY_OFFLINE), NULL);
 		g_object_unref(contact);
 		return;
 	}
@@ -433,23 +435,23 @@ e_book_backend_webdav_modify_contact(EBookBackend *backend,
 	if (status != 201 && status != 204) {
 		g_object_unref(contact);
 		if (status == 401 || status == 407) {
-			GNOME_Evolution_Addressbook_CallStatus res
-				= e_book_backend_handle_auth_request(webdav);
-			e_data_book_respond_remove_contacts(book, opid, res, NULL);
+			e_data_book_respond_remove_contacts (book, opid, webdav_handle_auth_request (webdav), NULL);
 			return;
 		}
 		/* data changed on server while we were editing */
 		if (status == 412) {
-			g_warning("contact on server changed -> not modifying");
 			/* too bad no special error code in evolution for this... */
-			e_data_book_respond_modify(book, opid,
-					GNOME_Evolution_Addressbook_OtherError, NULL);
+			e_data_book_respond_modify (book, opid,
+					e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR,
+						"Contact on server changed -> not modifying"),
+					NULL);
 			return;
 		}
 
-		g_warning("modify contact failed with http status: %d", status);
-		e_data_book_respond_modify(book, opid,
-				GNOME_Evolution_Addressbook_OtherError, NULL);
+		e_data_book_respond_modify (book, opid,
+				e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR,
+					"Modify contact failed with http status: %d", status),
+				NULL);
 		return;
 	}
 
@@ -470,8 +472,7 @@ e_book_backend_webdav_modify_contact(EBookBackend *backend,
 	}
 	e_book_backend_cache_add_contact(priv->cache, contact);
 
-	e_data_book_respond_modify(book, opid,
-			GNOME_Evolution_Addressbook_Success, contact);
+	e_data_book_respond_modify (book, opid, EDB_ERROR (SUCCESS), contact);
 
 	g_object_unref(contact);
 }
@@ -485,7 +486,7 @@ e_book_backend_webdav_get_contact(EBookBackend *backend, EDataBook *book,
 	EContact                  *contact;
 	gchar                      *vcard;
 
-	if (priv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
+	if (priv->mode == E_DATA_BOOK_MODE_LOCAL) {
 		contact = e_book_backend_cache_get_contact(priv->cache, uid);
 	} else {
 		contact = download_contact(webdav, uid);
@@ -497,14 +498,12 @@ e_book_backend_webdav_get_contact(EBookBackend *backend, EDataBook *book,
 	}
 
 	if (contact == NULL) {
-		e_data_book_respond_get_contact(book, opid,
-				GNOME_Evolution_Addressbook_OtherError, NULL);
+		e_data_book_respond_get_contact (book, opid, EDB_ERROR (CONTACT_NOT_FOUND), NULL);
 		return;
 	}
 
 	vcard = e_vcard_to_string(E_VCARD(contact), EVC_FORMAT_VCARD_30);
-	e_data_book_respond_get_contact(book, opid,
-			GNOME_Evolution_Addressbook_Success, vcard);
+	e_data_book_respond_get_contact (book, opid, EDB_ERROR (SUCCESS), vcard);
 	g_free(vcard);
 	g_object_unref(contact);
 }
@@ -656,8 +655,8 @@ send_propfind(EBookBackendWebdav *webdav)
 		"<propfind xmlns=\"DAV:\"><prop><getetag/></prop></propfind>";
 
 	message = soup_message_new(SOUP_METHOD_PROPFIND, priv->uri);
-	soup_message_headers_append(message->request_headers,
-				    "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "Connection", "close");
 	soup_message_headers_append(message->request_headers, "Depth", "1");
 	soup_message_set_request(message, "text/xml", SOUP_MEMORY_TEMPORARY,
 			(gchar *) request, strlen(request));
@@ -667,7 +666,151 @@ send_propfind(EBookBackendWebdav *webdav)
 	return message;
 }
 
-static GNOME_Evolution_Addressbook_CallStatus
+static xmlXPathObjectPtr
+xpath_eval (xmlXPathContextPtr ctx, const gchar *format, ...)
+{
+	xmlXPathObjectPtr  result;
+	va_list            args;
+	gchar              *expr;
+
+	if (ctx == NULL) {
+		return NULL;
+	}
+
+	va_start (args, format);
+	expr = g_strdup_vprintf (format, args);
+	va_end (args);
+
+	result = xmlXPathEvalExpression ((xmlChar *) expr, ctx);
+	g_free (expr);
+
+	if (result == NULL) {
+		return NULL;
+	}
+
+	if (result->type == XPATH_NODESET &&
+	    xmlXPathNodeSetIsEmpty (result->nodesetval)) {
+		xmlXPathFreeObject (result);
+		return NULL;
+	}
+
+	return result;
+}
+
+static gchar *
+xp_object_get_string (xmlXPathObjectPtr result)
+{
+	gchar *ret = NULL;
+
+	if (result == NULL)
+		return ret;
+
+	if (result->type == XPATH_STRING) {
+		ret = g_strdup ((gchar *) result->stringval);
+	}
+
+	xmlXPathFreeObject (result);
+	return ret;
+}
+
+static guint
+xp_object_get_status (xmlXPathObjectPtr result)
+{
+	gboolean res;
+	guint    ret = 0;
+
+	if (result == NULL)
+		return ret;
+
+	if (result->type == XPATH_STRING) {
+		res = soup_headers_parse_status_line ((gchar *) result->stringval, NULL, &ret, NULL);
+		if (!res) {
+			ret = 0;
+		}
+	}
+
+	xmlXPathFreeObject (result);
+	return ret;
+}
+
+static gboolean
+check_addressbook_changed (EBookBackendWebdav *webdav, gchar **new_ctag)
+{
+	gboolean res = TRUE;
+	const gchar *request = "<?xml version=\"1.0\" encoding=\"utf-8\"?><propfind xmlns=\"DAV:\"><prop><getctag/></prop></propfind>";
+	EBookBackendWebdavPrivate *priv;
+	SoupMessage *message;
+
+	g_return_val_if_fail (webdav != NULL, TRUE);
+	g_return_val_if_fail (new_ctag != NULL, TRUE);
+
+	*new_ctag = NULL;
+	priv = webdav->priv;
+
+	if (!priv->supports_getctag)
+		return TRUE;
+
+	priv->supports_getctag = FALSE;
+
+	message = soup_message_new (SOUP_METHOD_PROPFIND, priv->uri);
+	if (!message)
+		return TRUE;
+
+	soup_message_headers_append (message->request_headers, "User-Agent", USERAGENT);
+	soup_message_headers_append (message->request_headers, "Connection", "close");
+	soup_message_headers_append (message->request_headers, "Depth", "0");
+	soup_message_set_request (message, "text/xml", SOUP_MEMORY_TEMPORARY, (gchar *) request, strlen (request));
+	soup_session_send_message (priv->session, message);
+
+	if (message->status_code == 207 && message->response_body) {
+		xmlDocPtr xml;
+
+		xml = xmlReadMemory (message->response_body->data, message->response_body->length, NULL, NULL, XML_PARSE_NOWARNING);
+		if (xml) {
+			const gchar *GETCTAG_XPATH_STATUS = "string(/D:multistatus/D:response/D:propstat/D:prop/D:getctag/../../D:status)";
+			const gchar *GETCTAG_XPATH_VALUE = "string(/D:multistatus/D:response/D:propstat/D:prop/D:getctag)";
+			xmlXPathContextPtr xpctx;
+
+			xpctx = xmlXPathNewContext (xml);
+			xmlXPathRegisterNs (xpctx, (xmlChar *) "D", (xmlChar *) "DAV:");
+
+			if (xp_object_get_status (xpath_eval (xpctx, GETCTAG_XPATH_STATUS)) == 200) {
+				gchar *txt = xp_object_get_string (xpath_eval (xpctx, GETCTAG_XPATH_VALUE));
+
+				if (txt && *txt) {
+					gint len = strlen (txt);
+
+					if (*txt == '\"' && len > 2 && txt [len - 1] == '\"') {
+						/* dequote */
+						*new_ctag = g_strndup (txt + 1, len - 2);
+					} else {
+						*new_ctag = txt;
+						txt = NULL;
+					}
+
+					if (*new_ctag) {
+						const gchar *my_ctag;
+
+						my_ctag = e_file_cache_get_object (E_FILE_CACHE (priv->cache), WEBDAV_CTAG_KEY);
+						res = !my_ctag || !g_str_equal (my_ctag, *new_ctag);
+						priv->supports_getctag = TRUE;
+					}
+				}
+
+				g_free (txt);
+			}
+
+			xmlXPathFreeContext (xpctx);
+			xmlFreeDoc (xml);
+		}
+	}
+
+	g_object_unref (message);
+
+	return res;
+}
+
+static GError *
 download_contacts(EBookBackendWebdav *webdav, EFlag *running,
                   EDataBookView *book_view)
 {
@@ -680,6 +823,25 @@ download_contacts(EBookBackendWebdav *webdav, EFlag *running,
 	response_element_t        *next;
 	gint                        count;
 	gint                        i;
+	gchar                     *new_ctag = NULL;
+
+	if (!check_addressbook_changed (webdav, &new_ctag)) {
+		if (book_view) {
+			GList *contact_list, *cl;
+
+			contact_list = e_book_backend_cache_get_contacts (priv->cache, e_data_book_view_get_card_query (book_view));
+			for (cl = contact_list; cl != NULL; cl = g_list_next (cl)) {
+				EContact *contact = cl->data;
+
+				e_data_book_view_notify_update (book_view, contact);
+
+				g_object_unref (contact);
+			}
+			g_list_free (contact_list);
+		}
+		g_free (new_ctag);
+		return EDB_ERROR (SUCCESS);
+	}
 
 	if (book_view != NULL) {
 		e_data_book_view_notify_status_message(book_view,
@@ -690,23 +852,26 @@ download_contacts(EBookBackendWebdav *webdav, EFlag *running,
 	status  = message->status_code;
 
 	if (status == 401 || status == 407) {
-		GNOME_Evolution_Addressbook_CallStatus res
-			= e_book_backend_handle_auth_request(webdav);
 		g_object_unref(message);
-		e_data_book_view_unref(book_view);
-		return res;
+		g_free (new_ctag);
+		if (book_view)
+			e_data_book_view_unref(book_view);
+		return webdav_handle_auth_request (webdav);
 	}
 	if (status != 207) {
-		g_warning("PROPFIND on webdav failed with http status %d", status);
 		g_object_unref(message);
-		e_data_book_view_unref(book_view);
-		return GNOME_Evolution_Addressbook_OtherError;
+		g_free (new_ctag);
+		if (book_view)
+			e_data_book_view_unref(book_view);
+		return e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR, "PROPFIND on webdav failed with http status %d", status);
 	}
 	if (message->response_body == NULL) {
-		g_warning("No response body in webdav PROPEFIND result");
+		g_warning("No response body in webdav PROPFIND result");
 		g_object_unref(message);
-		e_data_book_view_unref(book_view);
-		return GNOME_Evolution_Addressbook_OtherError;
+		g_free (new_ctag);
+		if (book_view)
+			e_data_book_view_unref(book_view);
+		return e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR, "No response body in webdav PROPFIND result");
 	}
 
 	/* parse response */
@@ -789,7 +954,13 @@ download_contacts(EBookBackendWebdav *webdav, EFlag *running,
 	xmlFreeTextReader(reader);
 	g_object_unref(message);
 
-	return GNOME_Evolution_Addressbook_Success;
+	if (new_ctag) {
+		if (!e_file_cache_replace_object (E_FILE_CACHE (priv->cache), WEBDAV_CTAG_KEY, new_ctag))
+			e_file_cache_add_object (E_FILE_CACHE (priv->cache), WEBDAV_CTAG_KEY, new_ctag);
+	}
+	g_free (new_ctag);
+
+	return EDB_ERROR (SUCCESS);
 }
 
 static gpointer
@@ -798,7 +969,7 @@ book_view_thread(gpointer data)
 	EDataBookView                          *book_view = data;
 	WebdavBackendSearchClosure             *closure   = get_closure(book_view);
 	EBookBackendWebdav                     *webdav    = closure->webdav;
-	GNOME_Evolution_Addressbook_CallStatus  status;
+	GError *error;
 
 	e_flag_set(closure->running);
 
@@ -806,12 +977,15 @@ book_view_thread(gpointer data)
 	 * it's stopped */
 	e_data_book_view_ref(book_view);
 
-	status = download_contacts(webdav, closure->running, book_view);
+	error = download_contacts (webdav, closure->running, book_view);
 
 	e_data_book_view_unref(book_view);
 
 	/* report back status if query wasn't aborted */
-	e_data_book_view_notify_complete(book_view, status);
+	e_data_book_view_notify_complete (book_view, error);
+
+	if (error)
+		g_error_free (error);
 	return NULL;
 }
 
@@ -822,7 +996,7 @@ e_book_backend_webdav_start_book_view(EBookBackend *backend,
 	EBookBackendWebdav        *webdav = E_BOOK_BACKEND_WEBDAV(backend);
 	EBookBackendWebdavPrivate *priv   = webdav->priv;
 
-	if (priv->mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
+	if (priv->mode == E_DATA_BOOK_MODE_REMOTE) {
 		WebdavBackendSearchClosure *closure
 			= init_closure(book_view, E_BOOK_BACKEND_WEBDAV(backend));
 
@@ -841,8 +1015,7 @@ e_book_backend_webdav_start_book_view(EBookBackend *backend,
 			g_object_unref(contact);
 		}
 		g_list_free(contacts);
-		e_data_book_view_notify_complete(book_view,
-				GNOME_Evolution_Addressbook_Success);
+		e_data_book_view_notify_complete (book_view, NULL /* Success */);
 	}
 }
 
@@ -854,7 +1027,7 @@ e_book_backend_webdav_stop_book_view(EBookBackend *backend,
 	WebdavBackendSearchClosure *closure;
 	gboolean                    need_join;
 
-	if (webdav->priv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL)
+	if (webdav->priv->mode == E_DATA_BOOK_MODE_LOCAL)
 		return;
 
 	closure = get_closure(book_view);
@@ -879,13 +1052,12 @@ e_book_backend_webdav_get_contact_list(EBookBackend *backend, EDataBook *book,
 	GList                     *vcard_list;
 	GList                     *c;
 
-	if (priv->mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
+	if (priv->mode == E_DATA_BOOK_MODE_REMOTE) {
 		/* make sure the cache is up to date */
-		GNOME_Evolution_Addressbook_CallStatus status =
-			download_contacts(webdav, NULL, NULL);
+		GError *error = download_contacts (webdav, NULL, NULL);
 
-		if (status != GNOME_Evolution_Addressbook_Success) {
-			e_data_book_respond_get_contact_list(book, opid, status, NULL);
+		if (error) {
+			e_data_book_respond_get_contact_list (book, opid, error, NULL);
 			return;
 		}
 	}
@@ -902,8 +1074,7 @@ e_book_backend_webdav_get_contact_list(EBookBackend *backend, EDataBook *book,
 	}
 	g_list_free(contact_list);
 
-	e_data_book_respond_get_contact_list(book, opid,
-			GNOME_Evolution_Addressbook_Success, vcard_list);
+	e_data_book_respond_get_contact_list (book, opid, EDB_ERROR (SUCCESS), vcard_list);
 }
 
 static void
@@ -931,11 +1102,9 @@ e_book_backend_webdav_authenticate_user(EBookBackend *backend, EDataBook *book,
 		g_free(priv->password);
 		priv->password = NULL;
 
-		e_data_book_respond_authenticate_user(book, opid,
-				GNOME_Evolution_Addressbook_AuthenticationFailed);
+		e_data_book_respond_authenticate_user (book, opid, EDB_ERROR (AUTHENTICATION_FAILED));
 	} else {
-		e_data_book_respond_authenticate_user(book, opid,
-				GNOME_Evolution_Addressbook_Success);
+		e_data_book_respond_authenticate_user (book, opid, EDB_ERROR (SUCCESS));
 	}
 }
 
@@ -951,8 +1120,7 @@ e_book_backend_webdav_get_supported_fields(EBookBackend *backend,
 		fields = g_list_append(fields, g_strdup(e_contact_field_name(i)));
 	}
 
-	e_data_book_respond_get_supported_fields(book, opid,
-			GNOME_Evolution_Addressbook_Success, fields);
+	e_data_book_respond_get_supported_fields (book, opid, EDB_ERROR (SUCCESS), fields);
 	g_list_foreach(fields, (GFunc) g_free, NULL);
 	g_list_free(fields);
 }
@@ -965,8 +1133,7 @@ e_book_backend_webdav_get_supported_auth_methods(EBookBackend *backend,
 
 	auth_methods = g_list_append(auth_methods, g_strdup("plain/password"));
 
-	e_data_book_respond_get_supported_auth_methods(book, opid,
-			GNOME_Evolution_Addressbook_Success, auth_methods);
+	e_data_book_respond_get_supported_auth_methods (book, opid, EDB_ERROR (SUCCESS), auth_methods);
 
 	g_list_foreach(auth_methods, (GFunc) g_free, NULL);
 	g_list_free(auth_methods);
@@ -982,8 +1149,7 @@ e_book_backend_webdav_get_required_fields(EBookBackend *backend,
 	field_name = e_contact_field_name(E_CONTACT_FILE_AS);
 	fields     = g_list_append(fields , g_strdup(field_name));
 
-	e_data_book_respond_get_supported_fields(book, opid,
-			GNOME_Evolution_Addressbook_Success, fields);
+	e_data_book_respond_get_supported_fields (book, opid, EDB_ERROR (SUCCESS), fields);
 	g_list_free (fields);
 }
 
@@ -1020,9 +1186,9 @@ proxy_settings_changed (EProxy *proxy, gpointer user_data)
 	g_object_set (priv->session, SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
 }
 
-static GNOME_Evolution_Addressbook_CallStatus
+static void
 e_book_backend_webdav_load_source(EBookBackend *backend,
-                                  ESource *source, gboolean only_if_exists)
+                                  ESource *source, gboolean only_if_exists, GError **perror)
 {
 	EBookBackendWebdav        *webdav = E_BOOK_BACKEND_WEBDAV(backend);
 	EBookBackendWebdavPrivate *priv   = webdav->priv;
@@ -1032,34 +1198,39 @@ e_book_backend_webdav_load_source(EBookBackend *backend,
 	SoupSession               *session;
 	SoupURI                   *suri;
 
+	/* will try fetch ctag for the first time, if it fails then sets this to FALSE */
+	priv->supports_getctag = TRUE;
+
 	uri = e_source_get_uri(source);
 	if (uri == NULL) {
-		g_warning("no uri given for addressbook");
-		return  GNOME_Evolution_Addressbook_OtherError;
+		g_propagate_error (perror, EDB_ERROR_EX (OTHER_ERROR, "No uri given for addressbook"));
+		return;
 	}
 
 	suri = soup_uri_new (uri);
 	g_free (uri);
 
 	if (!suri) {
-		g_warning ("invalid uri given for addressbook");
-		return  GNOME_Evolution_Addressbook_OtherError;
+		g_propagate_error (perror, EDB_ERROR_EX (OTHER_ERROR, "Invalid uri given for addressbook"));
+		return;
 	}
 
 	offline = e_source_get_property(source, "offline_sync");
 	if (offline && g_str_equal(offline, "1"))
 		priv->marked_for_offline = TRUE;
 
-	if (priv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL
+	if (priv->mode == E_DATA_BOOK_MODE_LOCAL
 			&& !priv->marked_for_offline ) {
 		soup_uri_free (suri);
-		return GNOME_Evolution_Addressbook_OfflineUnavailable;
+		g_propagate_error (perror, EDB_ERROR (OFFLINE_UNAVAILABLE));
+		return;
 	}
 
 	if (!suri->scheme || !g_str_equal (suri->scheme, "webdav")) {
 		/* the book is not for us */
 		soup_uri_free (suri);
-		return GNOME_Evolution_Addressbook_OtherError;
+		g_propagate_error (perror, EDB_ERROR_EX (OTHER_ERROR, "Not a webdav uri"));
+		return;
 	}
 
 	use_ssl = e_source_get_property (source, "use_ssl");
@@ -1095,7 +1266,8 @@ e_book_backend_webdav_load_source(EBookBackend *backend,
 	priv->uri = soup_uri_to_string (suri, FALSE);
 	if (!priv->uri) {
 		soup_uri_free (suri);
-		return GNOME_Evolution_Addressbook_OtherError;
+		g_propagate_error (perror, EDB_ERROR_EX (OTHER_ERROR, "Cannot transform SoupURI to string"));
+		return;
 	}
 
 	priv->cache = e_book_backend_cache_new(priv->uri);
@@ -1118,20 +1290,18 @@ e_book_backend_webdav_load_source(EBookBackend *backend,
 	e_book_backend_notify_writable(backend, TRUE);
 
 	soup_uri_free (suri);
-
-	return GNOME_Evolution_Addressbook_Success;
 }
 
 static void
 e_book_backend_webdav_remove(EBookBackend *backend,	EDataBook *book,
 		guint32 opid)
 {
-	e_data_book_respond_remove(book, opid, GNOME_Evolution_Addressbook_Success);
+	e_data_book_respond_remove (book, opid, EDB_ERROR (SUCCESS));
 }
 
 static void
 e_book_backend_webdav_set_mode(EBookBackend *backend,
-                               GNOME_Evolution_Addressbook_BookMode mode)
+                               EDataBookMode mode)
 {
 	EBookBackendWebdav *webdav = E_BOOK_BACKEND_WEBDAV(backend);
 
@@ -1141,11 +1311,11 @@ e_book_backend_webdav_set_mode(EBookBackend *backend,
 	if (!e_book_backend_is_loaded(backend))
 		return;
 
-	if (mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
+	if (mode == E_DATA_BOOK_MODE_LOCAL) {
 		e_book_backend_set_is_writable(backend, FALSE);
 		e_book_backend_notify_writable(backend, FALSE);
 		e_book_backend_notify_connection_status(backend, FALSE);
-	} else if (mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
+	} else if (mode == E_DATA_BOOK_MODE_REMOTE) {
 		e_book_backend_set_is_writable(backend, TRUE);
 		e_book_backend_notify_writable(backend, TRUE);
 		e_book_backend_notify_connection_status(backend, TRUE);
@@ -1158,10 +1328,10 @@ e_book_backend_webdav_get_static_capabilities(EBookBackend *backend)
 	return g_strdup("net,do-initial-query,contact-lists");
 }
 
-static GNOME_Evolution_Addressbook_CallStatus
-e_book_backend_webdav_cancel_operation(EBookBackend *backend, EDataBook *book)
+static void
+e_book_backend_webdav_cancel_operation (EBookBackend *backend, EDataBook *book, GError **perror)
 {
-	return GNOME_Evolution_Addressbook_CouldNotCancel;
+	g_propagate_error (perror, EDB_ERROR (COULD_NOT_CANCEL));
 }
 
 EBookBackend *
