@@ -34,6 +34,8 @@
 
 #define CRLF "\r\n"
 
+G_DEFINE_TYPE (EVCard, e_vcard, G_TYPE_OBJECT)
+
 /** Encoding used in v-card
  *  Note: v-card spec defines additional 7BIT 8BIT and X- encoding
  */
@@ -48,8 +50,8 @@ struct _EVCardPrivate {
 };
 
 struct _EVCardAttribute {
-	char  *group;
-	char  *name;
+	gchar  *group;
+	gchar  *name;
 	GList *params; /* EVCardParam */
 	GList *values;
 	GList *decoded_values;
@@ -58,8 +60,8 @@ struct _EVCardAttribute {
 };
 
 struct _EVCardAttributeParam {
-	char     *name;
-	GList    *values;  /* GList of char*'s*/
+	gchar     *name;
+	GList    *values;  /* GList of gchar *'s*/
 };
 
 static GObjectClass *parent_class;
@@ -100,43 +102,17 @@ e_vcard_init (EVCard *evc)
 	evc->priv = g_new0 (EVCardPrivate, 1);
 }
 
-GType
-e_vcard_get_type (void)
-{
-	static GType vcard_type = 0;
-
-	if (!vcard_type) {
-		static const GTypeInfo vcard_info =  {
-			sizeof (EVCardClass),
-			NULL,           /* base_init */
-			NULL,           /* base_finalize */
-			(GClassInitFunc) e_vcard_class_init,
-			NULL,           /* class_finalize */
-			NULL,           /* class_data */
-			sizeof (EVCard),
-			0,             /* n_preallocs */
-			(GInstanceInitFunc) e_vcard_init,
-		};
-
-		vcard_type = g_type_register_static (G_TYPE_OBJECT, "EVCard", &vcard_info, 0);
-	}
-
-	return vcard_type;
-}
-
-
-
 /*  Skip newline characters and return the next character.
  *  This function takes care of folding lines, skipping
  *  newline characters if found, taking care of equal characters
  *  and other strange things.
  */
-static char*
-skip_newline (char *str, gboolean quoted_printable)
+static gchar *
+skip_newline (gchar *str, gboolean quoted_printable)
 {
-	char *p;
-	char *next;
-	char *next2;
+	gchar *p;
+	gchar *next;
+	gchar *next2;
 	p = str;
 
 	/* -- swallow equal signs at end of line for quoted printable */
@@ -176,19 +152,18 @@ skip_newline (char *str, gboolean quoted_printable)
 	return p;
 }
 
-
 /* skip forward until we hit the CRLF, or \0 */
 static void
-skip_to_next_line (char **p)
+skip_to_next_line (gchar **p)
 {
-	char *lp;
+	gchar *lp;
 	lp = *p;
 
 	while (*lp != '\n' && *lp != '\r' && *lp != '\0')
 		lp = g_utf8_next_char (lp);
 
 	/* -- skip over the endline */
-	while( *lp == '\r' || *lp == '\n' ) {
+	while (*lp == '\r' || *lp == '\n') {
 		lp = g_utf8_next_char (lp);
 	}
 
@@ -198,15 +173,15 @@ skip_to_next_line (char **p)
 /* skip forward until we hit a character in @s, CRLF, or \0.  leave *p
    pointing at the character that causes us to stop */
 static void
-skip_until (char **p, char *s)
+skip_until (gchar **p, const gchar *s)
 {
-	char *lp;
+	gchar *lp;
 
 	lp = *p;
 
 	while (*lp != '\r' && *lp != '\0') {
 		gboolean s_matches = FALSE;
-		char *ls;
+		const gchar *ls;
 		for (ls = s; *ls; ls = g_utf8_next_char (ls)) {
 			if (g_utf8_get_char (ls) == g_utf8_get_char (lp)) {
 				s_matches = TRUE;
@@ -223,23 +198,23 @@ skip_until (char **p, char *s)
 }
 
 static void
-read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable, const char *charset)
+read_attribute_value (EVCardAttribute *attr, gchar **p, gboolean quoted_printable, const gchar *charset)
 {
-	char *lp = *p;
+	gchar *lp = *p;
 	GString *str;
 
 	/* read in the value */
 	str = g_string_new ("");
-	for( lp =  skip_newline( *p, quoted_printable );
+	for (lp =  skip_newline( *p, quoted_printable);
 	     *lp != '\n' && *lp != '\r' && *lp != '\0';
 	     lp = skip_newline( lp, quoted_printable ) ) {
 
 		if (*lp == '=' && quoted_printable) {
-			char a, b;
+			gchar a, b;
 			if ((a = *(++lp)) == '\0') break;
 			if ((b = *(++lp)) == '\0') break;
 			if (isxdigit(a) && isxdigit (b)) {
-				char c;
+				gchar c;
 
 				a = tolower (a);
 				b = tolower (b);
@@ -290,7 +265,7 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 		else if ((*lp == ';') ||
 			 (*lp == ',' && !g_ascii_strcasecmp (attr->name, "CATEGORIES"))) {
 			if (charset) {
-				char *tmp;
+				gchar *tmp;
 
 				tmp = g_convert (str->str, str->len, "UTF-8", charset, NULL, NULL, NULL);
 				if (tmp) {
@@ -310,7 +285,7 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 	}
 	if (str) {
 		if (charset) {
-			char *tmp;
+			gchar *tmp;
 
 			tmp = g_convert (str->str, str->len, "UTF-8", charset, NULL, NULL, NULL);
 			if (tmp) {
@@ -329,15 +304,15 @@ read_attribute_value (EVCardAttribute *attr, char **p, gboolean quoted_printable
 }
 
 static void
-read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printable, char **charset)
+read_attribute_params (EVCardAttribute *attr, gchar **p, gboolean *quoted_printable, gchar **charset)
 {
-	char *lp;
+	gchar *lp;
 	GString *str;
 	EVCardAttributeParam *param = NULL;
 	gboolean in_quote = FALSE;
 
 	str = g_string_new ("");
-	for( lp =  skip_newline( *p, *quoted_printable );
+	for (lp =  skip_newline( *p, *quoted_printable);
 	     *lp != '\n' && *lp != '\r' && *lp != '\0';
 	     lp = skip_newline( lp, *quoted_printable ) ) {
 
@@ -420,7 +395,7 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 			}
 			else {
 				if (str->len > 0) {
-					char *param_name;
+					const gchar *param_name;
 					if (!g_ascii_strcasecmp (str->str,
 								 "quoted-printable")) {
 						param_name = "ENCODING";
@@ -487,19 +462,19 @@ read_attribute_params (EVCardAttribute *attr, char **p, gboolean *quoted_printab
 /* reads an entire attribute from the input buffer, leaving p pointing
    at the start of the next line (past the \r\n) */
 static EVCardAttribute*
-read_attribute (char **p)
+read_attribute (gchar **p)
 {
-	char *attr_group = NULL;
-	char *attr_name = NULL;
+	gchar *attr_group = NULL;
+	gchar *attr_name = NULL;
 	EVCardAttribute *attr = NULL;
 	GString *str;
-	char *lp;
+	gchar *lp;
 	gboolean is_qp = FALSE;
-	char *charset = NULL;
+	gchar *charset = NULL;
 
 	/* first read in the group/name */
 	str = g_string_new ("");
-	for( lp =  skip_newline( *p, is_qp );
+	for (lp =  skip_newline( *p, is_qp);
 	     *lp != '\n' && *lp != '\r' && *lp != '\0';
 	     lp = skip_newline( lp, is_qp ) ) {
 
@@ -594,10 +569,10 @@ make_valid_utf8 (const gchar *name)
 	gint remaining_bytes, valid_bytes;
 
 	string = NULL;
-  	remainder = name;
+	remainder = name;
 	remaining_bytes = strlen (name);
 
- 	while (remaining_bytes != 0) {
+	while (remaining_bytes != 0) {
 		if (g_utf8_validate (remainder, remaining_bytes, &invalid))
 			break;
 	      valid_bytes = invalid - remainder;
@@ -606,15 +581,15 @@ make_valid_utf8 (const gchar *name)
 			string = g_string_sized_new (remaining_bytes);
 
 		g_string_append_len (string, remainder, valid_bytes);
-	        /* append U+FFFD REPLACEMENT CHARACTER */
-	        g_string_append (string, "\357\277\275");
+		/* append U+FFFD REPLACEMENT CHARACTER */
+		g_string_append (string, "\357\277\275");
 
-	        remaining_bytes -= valid_bytes + 1;
-	        remainder = invalid + 1;
+		remaining_bytes -= valid_bytes + 1;
+		remainder = invalid + 1;
 	}
 
-  	if (string == NULL)
-    		return g_strdup (name);
+	if (string == NULL)
+		return g_strdup (name);
 
 	g_string_append (string, remainder);
 
@@ -628,16 +603,14 @@ make_valid_utf8 (const gchar *name)
  * try to return *something*.
  */
 static void
-parse (EVCard *evc, const char *str)
+parse (EVCard *evc, const gchar *str)
 {
-	char *buf ;
-	char *p;
+	gchar *buf;
+	gchar *p;
 	EVCardAttribute *attr;
 
 	buf = make_valid_utf8 (str);
 
-	//buf = fold_lines (buf);
-
 	d(printf ("BEFORE FOLDING:\n"));
 	d(printf (str));
 	d(printf ("\n\nAFTER FOLDING:\n"));
@@ -686,13 +659,13 @@ parse (EVCard *evc, const char *str)
  *
  * Escapes a string according to RFC2426, section 5.
  *
- * Return value: A newly allocated, escaped string.
+ * Returns: A newly allocated, escaped string.
  **/
-char*
-e_vcard_escape_string (const char *s)
+gchar *
+e_vcard_escape_string (const gchar *s)
 {
 	GString *str;
-	const char *p;
+	const gchar *p;
 
 	str = g_string_new ("");
 
@@ -731,13 +704,13 @@ e_vcard_escape_string (const char *s)
  *
  * Unescapes a string according to RFC2426, section 5.
  *
- * Return value: A newly allocated, unescaped string.
+ * Returns: A newly allocated, unescaped string.
  **/
-char*
-e_vcard_unescape_string (const char *s)
+gchar *
+e_vcard_unescape_string (const gchar *s)
 {
 	GString *str;
-	const char *p;
+	const gchar *p;
 
 	g_return_val_if_fail (s != NULL, NULL);
 
@@ -770,7 +743,7 @@ e_vcard_unescape_string (const char *s)
 }
 
 void
-e_vcard_construct (EVCard *evc, const char *str)
+e_vcard_construct (EVCard *evc, const gchar *str)
 {
 	g_return_if_fail (E_IS_VCARD (evc));
 	g_return_if_fail (str != NULL);
@@ -784,7 +757,7 @@ e_vcard_construct (EVCard *evc, const char *str)
  *
  * Creates a new, blank #EVCard.
  *
- * Return value: A new, blank #EVCard.
+ * Returns: A new, blank #EVCard.
  **/
 EVCard *
 e_vcard_new (void)
@@ -799,10 +772,10 @@ e_vcard_new (void)
  * Creates a new #EVCard from the passed-in string
  * representation.
  *
- * Return value: A new #EVCard.
+ * Returns: A new #EVCard.
  **/
 EVCard *
-e_vcard_new_from_string (const char *str)
+e_vcard_new_from_string (const gchar *str)
 {
 	EVCard *evc;
 
@@ -815,14 +788,14 @@ e_vcard_new_from_string (const char *str)
 	return evc;
 }
 
-static char*
+static gchar *
 e_vcard_to_string_vcard_21  (EVCard *evc)
 {
 	g_warning ("need to implement e_vcard_to_string_vcard_21");
 	return g_strdup ("");
 }
 
-static char*
+static gchar *
 e_vcard_to_string_vcard_30 (EVCard *evc)
 {
 	GList *l;
@@ -870,8 +843,8 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 			if (param->values) {
 				g_string_append_c (attr_str, '=');
 				for (v = param->values; v; v = v->next) {
-					char *value = v->data;
-					char *pval = value;
+					gchar *value = v->data;
+					gchar *pval = value;
 					gboolean quotes = FALSE;
 					while (*pval) {
 						if (!g_unichar_isalnum (g_utf8_get_char (pval))) {
@@ -882,7 +855,7 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 					}
 
 					if (quotes) {
-						int i;
+						gint i;
 
 						g_string_append_c (attr_str, '"');
 
@@ -907,15 +880,15 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		g_string_append_c (attr_str, ':');
 
 		for (v = attr->values; v; v = v->next) {
-			char *value = v->data;
-			char *escaped_value = NULL;
+			gchar *value = v->data;
+			gchar *escaped_value = NULL;
 
 			escaped_value = e_vcard_escape_string (value);
 
 			g_string_append (attr_str, escaped_value);
 			if (v->next) {
 				/* XXX toshok - i hate you, rfc 2426.
-				   why doesn't CATEGORIES use a ; like
+				   why doesn't CATEGORIES use a; like
 				   a normal list attribute? */
 				if (!g_ascii_strcasecmp (attr->name, "CATEGORIES"))
 					g_string_append_c (attr_str, ',');
@@ -970,9 +943,9 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
  * Exports @evc to a string representation, specified
  * by the @format argument.
  *
- * Return value: A newly allocated string representing the vcard.
+ * Returns: A newly allocated string representing the vcard.
  **/
-char*
+gchar *
 e_vcard_to_string (EVCard *evc, EVCardFormat format)
 {
 	g_return_val_if_fail (E_IS_VCARD (evc), NULL);
@@ -1000,7 +973,7 @@ e_vcard_dump_structure (EVCard *evc)
 {
 	GList *a;
 	GList *v;
-	int i;
+	gint i;
 
 	g_return_if_fail (E_IS_VCARD (evc));
 
@@ -1017,7 +990,7 @@ e_vcard_dump_structure (EVCard *evc)
 				printf ("    |   [%d] = %s", i,param->name);
 				printf ("(");
 				for (v = param->values; v; v = v->next) {
-					char *value = e_vcard_escape_string ((char*)v->data);
+					gchar *value = e_vcard_escape_string ((gchar *)v->data);
 					printf ("%s", value);
 					if (v->next)
 						printf (",");
@@ -1029,7 +1002,7 @@ e_vcard_dump_structure (EVCard *evc)
 		}
 		printf ("    +- values=\n");
 		for (v = attr->values, i = 0; v; v = v->next, i++) {
-			printf ("        [%d] = `%s'\n", i, (char*)v->data);
+			printf ("        [%d] = `%s'\n", i, (gchar *)v->data);
 		}
 	}
 }
@@ -1042,10 +1015,10 @@ e_vcard_dump_structure (EVCard *evc)
  * Creates a new #EVCardAttribute with the specified group and
  * attribute names.
  *
- * Return value: A new #EVCardAttribute.
+ * Returns: A new #EVCardAttribute.
  **/
 EVCardAttribute*
-e_vcard_attribute_new (const char *attr_group, const char *attr_name)
+e_vcard_attribute_new (const gchar *attr_group, const gchar *attr_name)
 {
 	EVCardAttribute *attr;
 
@@ -1084,7 +1057,7 @@ e_vcard_attribute_free (EVCardAttribute *attr)
  *
  * Makes a copy of @attr.
  *
- * Return value: A new #EVCardAttribute identical to @attr.
+ * Returns: A new #EVCardAttribute identical to @attr.
  **/
 EVCardAttribute*
 e_vcard_attribute_copy (EVCardAttribute *attr)
@@ -1117,7 +1090,7 @@ e_vcard_attribute_copy (EVCardAttribute *attr)
  * their group names.
  **/
 void
-e_vcard_remove_attributes (EVCard *evc, const char *attr_group, const char *attr_name)
+e_vcard_remove_attributes (EVCard *evc, const gchar *attr_group, const gchar *attr_name)
 {
 	GList *attr;
 
@@ -1162,12 +1135,78 @@ e_vcard_remove_attribute (EVCard *evc, EVCardAttribute *attr)
 	e_vcard_attribute_free (attr);
 }
 
+/**
+ * e_vcard_append_attribute:
+ * @evc: an #EVCard
+ * @attr: an #EVCardAttribute to append
+ *
+ * Appends @attr to @evc to the end of a list of attributes.
+ **/
+void
+e_vcard_append_attribute (EVCard *evc, EVCardAttribute *attr)
+{
+	g_return_if_fail (E_IS_VCARD (evc));
+	g_return_if_fail (attr != NULL);
+
+	evc->priv->attributes = g_list_append (evc->priv->attributes, attr);
+}
+
+/**
+ * e_vcard_append_attribute_with_value:
+ * @evcard: an #EVCard
+ * @attr: an #EVCardAttribute to append
+ * @value: a value to assign to the attribute
+ *
+ * Appends @attr to @evcard, setting it to @value.
+ * For attribute addition is used e_vcard_append_attribute().
+ **/
+void
+e_vcard_append_attribute_with_value (EVCard *evcard,
+				     EVCardAttribute *attr, const gchar *value)
+{
+	g_return_if_fail (E_IS_VCARD (evcard));
+	g_return_if_fail (attr != NULL);
+
+	e_vcard_attribute_add_value (attr, value);
+
+	e_vcard_append_attribute (evcard, attr);
+}
+
+/**
+ * e_vcard_append_attribute_with_values:
+ * @evcard: an @EVCard
+ * @attr: an #EVCardAttribute to append
+ * @Varargs: a %NULL-terminated list of values to assign to the attribute
+ *
+ * Appends @attr to @evcard, assigning the list of values to it.
+ * For attribute addition is used e_vcard_append_attribute().
+ **/
+void
+e_vcard_append_attribute_with_values (EVCard *evcard, EVCardAttribute *attr, ...)
+{
+	va_list ap;
+	gchar *v;
+
+	g_return_if_fail (E_IS_VCARD (evcard));
+	g_return_if_fail (attr != NULL);
+
+	va_start (ap, attr);
+
+	while ((v = va_arg (ap, gchar *))) {
+		e_vcard_attribute_add_value (attr, v);
+	}
+
+	va_end (ap);
+
+	e_vcard_append_attribute (evcard, attr);
+}
+
 /**
  * e_vcard_add_attribute:
  * @evc: an #EVCard
  * @attr: an #EVCardAttribute to add
  *
- * Adds @attr to @evc.
+ * Adds @attr to @evc. It's added to the beginning of a list of attributes.
  **/
 void
 e_vcard_add_attribute (EVCard *evc, EVCardAttribute *attr)
@@ -1184,11 +1223,12 @@ e_vcard_add_attribute (EVCard *evc, EVCardAttribute *attr)
  * @attr: an #EVCardAttribute to add
  * @value: a value to assign to the attribute
  *
- * Adds @attr to @evcard, setting it to @value.
+ * Adds @attr to @evcard, setting it to @value. For attribute addition
+ * is used e_vcard_add_attribute().
  **/
 void
 e_vcard_add_attribute_with_value (EVCard *evcard,
-				  EVCardAttribute *attr, const char *value)
+				  EVCardAttribute *attr, const gchar *value)
 {
 	g_return_if_fail (E_IS_VCARD (evcard));
 	g_return_if_fail (attr != NULL);
@@ -1205,19 +1245,20 @@ e_vcard_add_attribute_with_value (EVCard *evcard,
  * @Varargs: a %NULL-terminated list of values to assign to the attribute
  *
  * Adds @attr to @evcard, assigning the list of values to it.
+ * For attribute addition is used e_vcard_add_attribute().
  **/
 void
 e_vcard_add_attribute_with_values (EVCard *evcard, EVCardAttribute *attr, ...)
 {
 	va_list ap;
-	char *v;
+	gchar *v;
 
 	g_return_if_fail (E_IS_VCARD (evcard));
 	g_return_if_fail (attr != NULL);
 
 	va_start (ap, attr);
 
-	while ((v = va_arg (ap, char*))) {
+	while ((v = va_arg (ap, gchar *))) {
 		e_vcard_attribute_add_value (attr, v);
 	}
 
@@ -1234,7 +1275,7 @@ e_vcard_add_attribute_with_values (EVCard *evcard, EVCardAttribute *attr, ...)
  * Adds @value to @attr's list of values.
  **/
 void
-e_vcard_attribute_add_value (EVCardAttribute *attr, const char *value)
+e_vcard_attribute_add_value (EVCardAttribute *attr, const gchar *value)
 {
 	g_return_if_fail (attr != NULL);
 
@@ -1251,7 +1292,7 @@ e_vcard_attribute_add_value (EVCardAttribute *attr, const char *value)
  * adds it to @attr's list of values.
  **/
 void
-e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, int len)
+e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const gchar *value, gint len)
 {
 	g_return_if_fail (attr != NULL);
 
@@ -1260,7 +1301,7 @@ e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, i
 		g_warning ("can't add_value_decoded with an attribute using RAW encoding.  you must set the ENCODING parameter first");
 		break;
 	case EVC_ENCODING_BASE64: {
-		char *b64_data = g_base64_encode (value, len);
+		gchar *b64_data = g_base64_encode ((guchar *) value, len);
 		GString *decoded = g_string_new_len (value, len);
 
 		/* make sure the decoded list is up to date */
@@ -1291,13 +1332,13 @@ e_vcard_attribute_add_values (EVCardAttribute *attr,
 			      ...)
 {
 	va_list ap;
-	char *v;
+	gchar *v;
 
 	g_return_if_fail (attr != NULL);
 
 	va_start (ap, attr);
 
-	while ((v = va_arg (ap, char*))) {
+	while ((v = va_arg (ap, gchar *))) {
 		e_vcard_attribute_add_value (attr, v);
 	}
 
@@ -1338,7 +1379,7 @@ e_vcard_attribute_remove_values (EVCardAttribute *attr)
  * Removes from the value list in @attr the value @s.
  **/
 void
-e_vcard_attribute_remove_value (EVCardAttribute *attr, const char *s)
+e_vcard_attribute_remove_value (EVCardAttribute *attr, const gchar *s)
 {
 	GList *l;
 
@@ -1360,10 +1401,10 @@ e_vcard_attribute_remove_value (EVCardAttribute *attr, const char *s)
  *
  * Removes the parameter @param_name from the attribute @attr.
  *
- * Since 1.11.3.
+ * Since: 1.12
  */
 void
-e_vcard_attribute_remove_param (EVCardAttribute *attr, const char *param_name)
+e_vcard_attribute_remove_param (EVCardAttribute *attr, const gchar *param_name)
 {
 	GList *l;
 	EVCardAttributeParam *param;
@@ -1408,10 +1449,10 @@ e_vcard_attribute_remove_params (EVCardAttribute *attr)
  *
  * Creates a new parameter named @name.
  *
- * Return value: A new #EVCardAttributeParam.
+ * Returns: A new #EVCardAttributeParam.
  **/
 EVCardAttributeParam*
-e_vcard_attribute_param_new (const char *name)
+e_vcard_attribute_param_new (const gchar *name)
 {
 	EVCardAttributeParam *param = g_slice_new (EVCardAttributeParam);
 	param->values = NULL;
@@ -1444,7 +1485,7 @@ e_vcard_attribute_param_free (EVCardAttributeParam *param)
  *
  * Makes a copy of @param.
  *
- * Return value: a new #EVCardAttributeParam identical to @param.
+ * Returns: a new #EVCardAttributeParam identical to @param.
  **/
 EVCardAttributeParam*
 e_vcard_attribute_param_copy (EVCardAttributeParam *param)
@@ -1480,7 +1521,7 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
 {
 	gboolean contains;
 	GList *params, *p;
-	const char *par_name;
+	const gchar *par_name;
 
 	g_return_if_fail (attr != NULL);
 	g_return_if_fail (param != NULL);
@@ -1501,14 +1542,14 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
 			vals = param->values;
 
 			for (v = vals; v; v = v->next) {
-				const char *my_value;
+				const gchar *my_value;
 				GList *vals2,*v2;
 
-				my_value = (const char *)v->data;
+				my_value = (const gchar *)v->data;
 				vals2 = param2->values;
 
 				for (v2 = vals2; v2; v2 = v2->next) {
-					if (g_ascii_strcasecmp ((const char *)v2->data, my_value) == 0) {
+					if (g_ascii_strcasecmp ((const gchar *)v2->data, my_value) == 0) {
 						break;
 					}
 				}
@@ -1540,14 +1581,14 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
 		}
 
 		if (param->values && param->values->data) {
-			if (!g_ascii_strcasecmp ((char*)param->values->data, "b") ||
-			    !g_ascii_strcasecmp ((char*)param->values->data, "BASE64"))
+			if (!g_ascii_strcasecmp ((gchar *)param->values->data, "b") ||
+			    !g_ascii_strcasecmp ((gchar *)param->values->data, "BASE64"))
 				attr->encoding = EVC_ENCODING_BASE64;
-			else if (!g_ascii_strcasecmp ((char*)param->values->data, EVC_QUOTEDPRINTABLE))
+			else if (!g_ascii_strcasecmp ((gchar *)param->values->data, EVC_QUOTEDPRINTABLE))
 				attr->encoding = EVC_ENCODING_QP;
 			else {
 				g_warning ("Unknown value `%s' for ENCODING parameter.  values will be treated as raw",
-					   (char*)param->values->data);
+					   (gchar *)param->values->data);
 			}
 
 			attr->encoding_set = TRUE;
@@ -1571,7 +1612,7 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
  **/
 void
 e_vcard_attribute_param_add_value (EVCardAttributeParam *param,
-				   const char *value)
+				   const gchar *value)
 {
 	g_return_if_fail (param != NULL);
 
@@ -1590,13 +1631,13 @@ e_vcard_attribute_param_add_values (EVCardAttributeParam *param,
 				    ...)
 {
 	va_list ap;
-	char *v;
+	gchar *v;
 
 	g_return_if_fail (param != NULL);
 
 	va_start (ap, param);
 
-	while ((v = va_arg (ap, char*))) {
+	while ((v = va_arg (ap, gchar *))) {
 		e_vcard_attribute_param_add_value (param, v);
 	}
 
@@ -1613,7 +1654,7 @@ e_vcard_attribute_param_add_values (EVCardAttributeParam *param,
  **/
 void
 e_vcard_attribute_add_param_with_value (EVCardAttribute *attr,
-					EVCardAttributeParam *param, const char *value)
+					EVCardAttributeParam *param, const gchar *value)
 {
 	g_return_if_fail (attr != NULL);
 	g_return_if_fail (param != NULL);
@@ -1637,14 +1678,14 @@ e_vcard_attribute_add_param_with_values (EVCardAttribute *attr,
 					 EVCardAttributeParam *param, ...)
 {
 	va_list ap;
-	char *v;
+	gchar *v;
 
 	g_return_if_fail (attr != NULL);
 	g_return_if_fail (param != NULL);
 
 	va_start (ap, param);
 
-	while ((v = va_arg (ap, char*))) {
+	while ((v = va_arg (ap, gchar *))) {
 		e_vcard_attribute_param_add_value (param, v);
 	}
 
@@ -1678,7 +1719,7 @@ e_vcard_attribute_param_remove_values (EVCardAttributeParam *param)
  * Removes the value @s from the parameter @param_name on the attribute @attr.
  **/
 void
-e_vcard_attribute_remove_param_value (EVCardAttribute *attr, const char *param_name, const char *s)
+e_vcard_attribute_remove_param_value (EVCardAttribute *attr, const gchar *param_name, const gchar *s)
 {
 	GList *l, *params;
 	EVCardAttributeParam *param;
@@ -1715,7 +1756,7 @@ e_vcard_attribute_remove_param_value (EVCardAttribute *attr, const char *param_n
  * Gets the list of attributes from @evcard. The list and its
  * contents are owned by @evcard, and must not be freed.
  *
- * Return value: A list of attributes of type #EVCardAttribute.
+ * Returns: A list of attributes of type #EVCardAttribute.
  **/
 GList*
 e_vcard_get_attributes (EVCard *evcard)
@@ -1734,11 +1775,11 @@ e_vcard_get_attributes (EVCard *evcard)
  * @evcard and should not be freed. If the attribute does not exist, #NULL is
  * returned.
  *
- * Return value: An #EVCardAttribute if found, or #NULL.
+ * Returns: An #EVCardAttribute if found, or #NULL.
  **/
 EVCardAttribute *
 e_vcard_get_attribute (EVCard     *evc,
-		       const char *name)
+		       const gchar *name)
 {
         GList *attrs, *l;
 
@@ -1762,9 +1803,9 @@ e_vcard_get_attribute (EVCard     *evc,
  *
  * Gets the group name of @attr.
  *
- * Return value: The attribute's group name.
+ * Returns: The attribute's group name.
  **/
-const char*
+const gchar *
 e_vcard_attribute_get_group (EVCardAttribute *attr)
 {
 	g_return_val_if_fail (attr != NULL, NULL);
@@ -1778,9 +1819,9 @@ e_vcard_attribute_get_group (EVCardAttribute *attr)
  *
  * Gets the name of @attr.
  *
- * Return value: The attribute's name.
+ * Returns: The attribute's name.
  **/
-const char*
+const gchar *
 e_vcard_attribute_get_name (EVCardAttribute *attr)
 {
 	g_return_val_if_fail (attr != NULL, NULL);
@@ -1795,7 +1836,7 @@ e_vcard_attribute_get_name (EVCardAttribute *attr)
  * Gets the list of values from @attr. The list and its
  * contents are owned by @attr, and must not be freed.
  *
- * Return value: A list of string values.
+ * Returns: A list of string values.
  **/
 GList*
 e_vcard_attribute_get_values (EVCardAttribute *attr)
@@ -1813,7 +1854,7 @@ e_vcard_attribute_get_values (EVCardAttribute *attr)
  * necessary. The list and its contents are owned by @attr,
  * and must not be freed.
  *
- * Return value: A list of values of type #GString.
+ * Returns: A list of values of type #GString.
  **/
 GList*
 e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
@@ -1825,7 +1866,7 @@ e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 		switch (attr->encoding) {
 		case EVC_ENCODING_RAW:
 			for (l = attr->values; l; l = l->next)
-				attr->decoded_values = g_list_prepend (attr->decoded_values, g_string_new ((char*)l->data));
+				attr->decoded_values = g_list_prepend (attr->decoded_values, g_string_new ((gchar *)l->data));
 			attr->decoded_values = g_list_reverse (attr->decoded_values);
 			break;
 		case EVC_ENCODING_BASE64:
@@ -1834,7 +1875,7 @@ e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 				gsize len = 0;
 
 				decoded = g_base64_decode (l->data, &len);
-				attr->decoded_values = g_list_prepend (attr->decoded_values, g_string_new_len (decoded, len));
+				attr->decoded_values = g_list_prepend (attr->decoded_values, g_string_new_len ((gchar *) decoded, len));
 				g_free (decoded);
 			}
 			attr->decoded_values = g_list_reverse (attr->decoded_values);
@@ -1854,7 +1895,7 @@ e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
  *
  * Checks if @attr has a single value.
  *
- * Return value: %TRUE if the attribute has exactly one value, %FALSE otherwise.
+ * Returns: %TRUE if the attribute has exactly one value, %FALSE otherwise.
  **/
 gboolean
 e_vcard_attribute_is_single_valued (EVCardAttribute *attr)
@@ -1874,9 +1915,9 @@ e_vcard_attribute_is_single_valued (EVCardAttribute *attr)
  *
  * Gets the value of a single-valued #EVCardAttribute, @attr.
  *
- * Return value: A newly allocated string representing the value.
+ * Returns: A newly allocated string representing the value.
  **/
-char*
+gchar *
 e_vcard_attribute_get_value (EVCardAttribute *attr)
 {
 	GList *values;
@@ -1888,7 +1929,7 @@ e_vcard_attribute_get_value (EVCardAttribute *attr)
 	if (!e_vcard_attribute_is_single_valued (attr))
 		g_warning ("e_vcard_attribute_get_value called on multivalued attribute");
 
-	return values ? g_strdup ((char*)values->data) : NULL;
+	return values ? g_strdup ((gchar *)values->data) : NULL;
 }
 
 /**
@@ -1900,7 +1941,7 @@ e_vcard_attribute_get_value (EVCardAttribute *attr)
  *
  * Note: this function seems currently to be unused. Could be removed.
  *
- * Return value: A newly allocated #GString representing the value.
+ * Returns: A newly allocated #GString representing the value.
  **/
 GString*
 e_vcard_attribute_get_value_decoded (EVCardAttribute *attr)
@@ -1928,10 +1969,10 @@ e_vcard_attribute_get_value_decoded (EVCardAttribute *attr)
  *
  * Checks if @attr has an #EVCardAttributeParam of the specified type.
  *
- * Return value: %TRUE if such a parameter exists, %FALSE otherwise.
+ * Returns: %TRUE if such a parameter exists, %FALSE otherwise.
  **/
 gboolean
-e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
+e_vcard_attribute_has_type (EVCardAttribute *attr, const gchar *typestr)
 {
 	GList *params;
 	GList *p;
@@ -1949,7 +1990,7 @@ e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
 			GList *v;
 
 			for (v = values; v; v = v->next) {
-				if (!g_ascii_strcasecmp ((char*)v->data, typestr))
+				if (!g_ascii_strcasecmp ((gchar *)v->data, typestr))
 					return TRUE;
 			}
 		}
@@ -1965,7 +2006,7 @@ e_vcard_attribute_has_type (EVCardAttribute *attr, const char *typestr)
  * Gets the list of parameters from @attr. The list and its
  * contents are owned by @attr, and must not be freed.
  *
- * Return value: A list of elements of type #EVCardAttributeParam.
+ * Returns: A list of elements of type #EVCardAttributeParam.
  **/
 GList*
 e_vcard_attribute_get_params (EVCardAttribute *attr)
@@ -1983,10 +2024,10 @@ e_vcard_attribute_get_params (EVCardAttribute *attr)
  * Gets the list of values for the paramater @name from @attr. The list and its
  * contents are owned by @attr, and must not be freed.
  *
- * Return value: A list of string elements representing the parameter's values.
+ * Returns: A list of string elements representing the parameter's values.
  **/
 GList *
-e_vcard_attribute_get_param (EVCardAttribute *attr, const char *name)
+e_vcard_attribute_get_param (EVCardAttribute *attr, const gchar *name)
 {
 	GList *params, *p;
 
@@ -2011,9 +2052,9 @@ e_vcard_attribute_get_param (EVCardAttribute *attr, const char *name)
  *
  * Gets the name of @param.
  *
- * Return value: The name of the parameter.
+ * Returns: The name of the parameter.
  **/
-const char*
+const gchar *
 e_vcard_attribute_param_get_name (EVCardAttributeParam *param)
 {
 	g_return_val_if_fail (param != NULL, NULL);
@@ -2028,7 +2069,7 @@ e_vcard_attribute_param_get_name (EVCardAttributeParam *param)
  * Gets the list of values from @param. The list and its
  * contents are owned by @param, and must not be freed.
  *
- * Return value: A list of string elements representing the parameter's values.
+ * Returns: A list of string elements representing the parameter's values.
  **/
 GList*
 e_vcard_attribute_param_get_values (EVCardAttributeParam *param)
