@@ -24,9 +24,12 @@
 #endif
 
 #include <string.h>
+#include <libedataserver/e-data-server-util.h>
 #include "e-book-backend-cache.h"
 #include "e-book-backend-sexp.h"
 
+G_DEFINE_TYPE (EBookBackendCache, e_book_backend_cache, E_TYPE_FILE_CACHE)
+
 struct _EBookBackendCachePrivate {
 	gchar *uri;
 };
@@ -42,24 +45,18 @@ static GObjectClass *parent_class = NULL;
 static gchar *
 get_filename_from_uri (const gchar *uri)
 {
+	const gchar *user_cache_dir;
 	gchar *mangled_uri, *filename;
-	gint i;
 
-	/* mangle the URI to not contain invalid characters */
-	mangled_uri = g_strdup (uri);
-	for (i = 0; i < strlen (mangled_uri); i++) {
-		switch (mangled_uri[i]) {
-		case ':' :
-		case '/' :
-			mangled_uri[i] = '_';
-		}
-	}
+	user_cache_dir = e_get_user_cache_dir ();
 
-	/* generate the file name */
-	filename = g_build_filename (g_get_home_dir (), ".evolution/cache/addressbook",
-				     mangled_uri, "cache.xml", NULL);
+	/* Mangle the URI to not contain invalid characters. */
+	mangled_uri = g_strdelimit (g_strdup (uri), ":/", '_');
+
+	filename = g_build_filename (
+		user_cache_dir, "addressbook",
+		mangled_uri, "cache.xml", NULL);
 
-	/* free memory */
 	g_free (mangled_uri);
 
 	return filename;
@@ -111,7 +108,6 @@ e_book_backend_cache_get_property (GObject *object, guint property_id, GValue *v
 	}
 }
 
-
 static void
 e_book_backend_cache_finalize (GObject *object)
 {
@@ -127,7 +123,6 @@ e_book_backend_cache_finalize (GObject *object)
 			priv->uri = NULL;
 		}
 
-
 		g_free (priv);
 		cache->priv = NULL;
 	}
@@ -191,29 +186,6 @@ e_book_backend_cache_init (EBookBackendCache *cache)
 
 }
 
-
-GType
-e_book_backend_cache_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static GTypeInfo info = {
-                        sizeof (EBookBackendCacheClass),
-                        (GBaseInitFunc) NULL,
-                        (GBaseFinalizeFunc) NULL,
-                        (GClassInitFunc) e_book_backend_cache_class_init,
-                        NULL, NULL,
-                        sizeof (EBookBackendCache),
-                        0,
-                        (GInstanceInitFunc) e_book_backend_cache_init,
-                };
-		type = g_type_register_static (E_TYPE_FILE_CACHE, "EBookBackendCache", &info, 0);
-	}
-
-	return type;
-}
-
 /**
  * e_book_backend_cache_new
  * @uri: URI of the backend to be cached.
@@ -221,7 +193,7 @@ e_book_backend_cache_get_type (void)
  * Creates a new #EBookBackendCache object, which implements a local
  * cache of #EContact objects, useful for remote backends.
  *
- * Return value: A new #EBookBackendCache.
+ * Returns: A new #EBookBackendCache.
  */
 EBookBackendCache *
 e_book_backend_cache_new (const gchar *uri)
@@ -242,7 +214,7 @@ e_book_backend_cache_new (const gchar *uri)
  * newly created, and must be unreffed by the caller when no longer
  * needed.
  *
- * Return value: A cached #EContact, or %NULL if @uid is not cached.
+ * Returns: A cached #EContact, or %NULL if @uid is not cached.
  **/
 EContact *
 e_book_backend_cache_get_contact (EBookBackendCache *cache, const gchar *uid)
@@ -259,7 +231,6 @@ e_book_backend_cache_get_contact (EBookBackendCache *cache, const gchar *uid)
 
 	}
 
-
 	return contact;
 }
 
@@ -270,7 +241,7 @@ e_book_backend_cache_get_contact (EBookBackendCache *cache, const gchar *uid)
  *
  * Adds @contact to @cache.
  *
- * Return value: %TRUE if the contact was cached successfully, %FALSE otherwise.
+ * Returns: %TRUE if the contact was cached successfully, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_cache_add_contact (EBookBackendCache *cache,
@@ -283,7 +254,6 @@ e_book_backend_cache_add_contact (EBookBackendCache *cache,
 
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_CACHE (cache), FALSE);
 
-
 	priv = cache->priv;
 
 	uid = e_contact_get_const (contact, E_CONTACT_UID);
@@ -306,7 +276,7 @@ e_book_backend_cache_add_contact (EBookBackendCache *cache,
  *
  * Removes the contact identified by @uid from @cache.
  *
- * Return value: %TRUE if the contact was found and removed, %FALSE otherwise.
+ * Returns: %TRUE if the contact was found and removed, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_cache_remove_contact (EBookBackendCache *cache,
@@ -321,14 +291,12 @@ e_book_backend_cache_remove_contact (EBookBackendCache *cache,
 
 	priv = cache->priv;
 
-
 	if (!e_file_cache_get_object (E_FILE_CACHE (cache), uid)) {
 		return FALSE;
 	}
 
 	retval = e_file_cache_remove_object (E_FILE_CACHE (cache), uid);
 
-
 	return retval;
 }
 
@@ -339,7 +307,7 @@ e_book_backend_cache_remove_contact (EBookBackendCache *cache,
  *
  * Checks if the contact identified by @uid exists in @cache.
  *
- * Return value: %TRUE if the cache contains the contact, %FALSE otherwise.
+ * Returns: %TRUE if the cache contains the contact, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_cache_check_contact (EBookBackendCache *cache, const gchar *uid)
@@ -368,7 +336,7 @@ e_book_backend_cache_check_contact (EBookBackendCache *cache, const gchar *uid)
  * When done with the list, the caller must unref the contacts and
  * free the list.
  *
- * Return value: A #GList of pointers to #EContact.
+ * Returns: A #GList of pointers to #EContact.
  **/
 GList *
 e_book_backend_cache_get_contacts (EBookBackendCache *cache, const gchar *query)
@@ -387,7 +355,6 @@ e_book_backend_cache_get_contacts (EBookBackendCache *cache, const gchar *query)
 			return NULL;
 	}
 
-
         lcache = l = e_file_cache_get_objects (E_FILE_CACHE (cache));
 
         for (; l != NULL; l = g_slist_next (l)) {
@@ -419,7 +386,7 @@ e_book_backend_cache_get_contacts (EBookBackendCache *cache, const gchar *query)
  * in @cache matching @query. When done with the array, the caller must
  * free the ID strings and the array.
  *
- * Return value: A #GPtrArray of pointers to contact ID strings.
+ * Returns: A #GPtrArray of pointers to contact ID strings.
  **/
 GPtrArray *
 e_book_backend_cache_search (EBookBackendCache *cache, const gchar *query)
@@ -446,7 +413,7 @@ e_book_backend_cache_search (EBookBackendCache *cache, const gchar *query)
  *
  * Checks if an #EBookBackendCache exists at @uri.
  *
- * Return value: %TRUE if cache exists, %FALSE if not.
+ * Returns: %TRUE if cache exists, %FALSE if not.
  **/
 gboolean
 e_book_backend_cache_exists (const gchar *uri)
@@ -484,7 +451,7 @@ e_book_backend_cache_set_populated (EBookBackendCache *cache)
  *
  * Checks if @cache is populated.
  *
- * Return value: %TRUE if @cache is populated, %FALSE otherwise.
+ * Returns: %TRUE if @cache is populated, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_cache_is_populated (EBookBackendCache *cache)
