@@ -31,6 +31,8 @@
 
 
 
+#define EDC_ERROR(_code) e_data_cal_create_error (_code, NULL)
+
 G_DEFINE_TYPE (ECalBackend, e_cal_backend, G_TYPE_OBJECT);
 
 /* Private part of the CalBackend structure */
@@ -78,13 +80,12 @@ enum props {
 /* Signal IDs */
 enum {
 	LAST_CLIENT_GONE,
-	OPENED,
-	REMOVED,
 	LAST_SIGNAL
 };
 static guint e_cal_backend_signals[LAST_SIGNAL];
 
 static void e_cal_backend_finalize (GObject *object);
+static void e_cal_backend_remove_client_private (ECalBackend *backend, EDataCal *cal, gboolean weak_unref);
 
 
 
@@ -224,28 +225,8 @@ e_cal_backend_class_init (ECalBackendClass *class)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
-	e_cal_backend_signals[OPENED] =
-		g_signal_new ("opened",
-			      G_TYPE_FROM_CLASS (class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (ECalBackendClass, opened),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__ENUM,
-			      G_TYPE_NONE, 1,
-			      G_TYPE_INT);
-	e_cal_backend_signals[REMOVED] =
-		g_signal_new ("removed",
-			      G_TYPE_FROM_CLASS (class),
-			      G_SIGNAL_RUN_FIRST,
-			      G_STRUCT_OFFSET (ECalBackendClass, removed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__ENUM,
-			      G_TYPE_NONE, 1,
-			      G_TYPE_INT);
 
 	class->last_client_gone = NULL;
-	class->opened = NULL;
-	class->obj_updated = NULL;
 
 	class->get_cal_address = NULL;
 	class->get_alarm_email_address = NULL;
@@ -324,7 +305,7 @@ e_cal_backend_finalize (GObject *object)
  *
  * Gets the #ESource associated with the given backend.
  *
- * Return value: The #ESource for the backend.
+ * Returns: The #ESource for the backend.
  */
 ESource *
 e_cal_backend_get_source (ECalBackend *backend)
@@ -346,7 +327,7 @@ e_cal_backend_get_source (ECalBackend *backend)
  * Queries the URI of a calendar backend, which must already have an open
  * calendar.
  *
- * Return value: The URI where the calendar is stored.
+ * Returns: The URI where the calendar is stored.
  **/
 const gchar *
 e_cal_backend_get_uri (ECalBackend *backend)
@@ -367,7 +348,7 @@ e_cal_backend_get_uri (ECalBackend *backend)
  *
  * Gets the kind of components the given backend stores.
  *
- * Return value: The kind of components for this backend.
+ * Returns: The kind of components for this backend.
  */
 icalcomponent_kind
 e_cal_backend_get_kind (ECalBackend *backend)
@@ -383,9 +364,10 @@ e_cal_backend_get_kind (ECalBackend *backend)
 }
 
 static void
-last_client_gone (ECalBackend *backend)
+cal_destroy_cb (gpointer data, GObject *where_cal_was)
 {
-	g_signal_emit (backend, e_cal_backend_signals[LAST_CLIENT_GONE], 0);
+	e_cal_backend_remove_client_private (E_CAL_BACKEND (data),
+					     (EDataCal *) where_cal_was, FALSE);
 }
 
 /**
@@ -408,29 +390,15 @@ e_cal_backend_add_client (ECalBackend *backend, EDataCal *cal)
 
 	priv = backend->priv;
 
-	/* TODO: Implement this? */
-#if 0
-	bonobo_object_set_immortal (BONOBO_OBJECT (cal), TRUE);
-
 	g_object_weak_ref (G_OBJECT (cal), cal_destroy_cb, backend);
 
-	ORBit_small_listen_for_broken (e_data_cal_get_listener (cal), G_CALLBACK (listener_died_cb), cal);
-#endif
-
 	g_mutex_lock (priv->clients_mutex);
 	priv->clients = g_list_append (priv->clients, cal);
 	g_mutex_unlock (priv->clients_mutex);
 }
 
-/**
- * e_cal_backend_remove_client:
- * @backend: An #ECalBackend object.
- * @cal: An #EDataCal object.
- *
- * Removes a client from the list of connected clients to the given backend.
- */
-void
-e_cal_backend_remove_client (ECalBackend *backend, EDataCal *cal)
+static void
+e_cal_backend_remove_client_private (ECalBackend *backend, EDataCal *cal, gboolean weak_unref)
 {
 	ECalBackendPrivate *priv;
 
@@ -444,6 +412,9 @@ e_cal_backend_remove_client (ECalBackend *backend, EDataCal *cal)
 
 	priv = backend->priv;
 
+	if (weak_unref)
+		g_object_weak_unref (G_OBJECT (cal), cal_destroy_cb, backend);
+
 	/* Disconnect */
 	g_mutex_lock (priv->clients_mutex);
 	priv->clients = g_list_remove (priv->clients, cal);
@@ -453,7 +424,20 @@ e_cal_backend_remove_client (ECalBackend *backend, EDataCal *cal)
 	 * it may decide whether to kill the backend or not.
 	 */
 	if (!priv->clients)
-		last_client_gone (backend);
+		g_signal_emit (backend, e_cal_backend_signals[LAST_CLIENT_GONE], 0);
+}
+
+/**
+ * e_cal_backend_remove_client:
+ * @backend: An #ECalBackend object.
+ * @cal: An #EDataCal object.
+ *
+ * Removes a client from the list of connected clients to the given backend.
+ */
+void
+e_cal_backend_remove_client (ECalBackend *backend, EDataCal *cal)
+{
+	e_cal_backend_remove_client_private (backend, cal, TRUE);
 }
 
 /**
@@ -484,7 +468,7 @@ e_cal_backend_add_query (ECalBackend *backend, EDataCalView *query)
  *
  * Gets the list of live queries being run on the given backend.
  *
- * Return value: The list of live queries.
+ * Returns: The list of live queries.
  */
 EList *
 e_cal_backend_get_queries (ECalBackend *backend)
@@ -501,6 +485,8 @@ e_cal_backend_get_queries (ECalBackend *backend)
  * @query: An #EDataCalView object, previously added with @ref e_cal_backend_add_query.
  *
  * Removes query from the list of live queries for the backend.
+ *
+ * Since: 2.24
  **/
 void
 e_cal_backend_remove_query (ECalBackend *backend, EDataCalView *query)
@@ -545,7 +531,7 @@ e_cal_backend_notify_readonly (ECalBackend *backend, gboolean read_only)
 		return;
 	}
 	for (l = priv->clients; l; l = l->next)
-		e_data_cal_notify_read_only (l->data, GNOME_Evolution_Calendar_Success, read_only);
+		e_data_cal_notify_read_only (l->data, NULL /* Success */, read_only);
 }
 
 void
@@ -557,7 +543,7 @@ e_cal_backend_notify_cal_address (ECalBackend *backend, EServerMethodContext con
 	priv = backend->priv;
 
 	for (l = priv->clients; l; l = l->next)
-		e_data_cal_notify_cal_address (l->data, context, GNOME_Evolution_Calendar_Success, address);
+		e_data_cal_notify_cal_address (l->data, context, NULL /* Success */, address);
 }
 
 /**
@@ -635,6 +621,25 @@ e_cal_backend_open (ECalBackend *backend, EDataCal *cal, EServerMethodContext co
 	(* CLASS (backend)->open) (backend, cal, context, only_if_exists, username, password);
 }
 
+/**
+ * e_cal_backend_refresh:
+ * @backend: A calendar backend.
+ * @cal: An #EDataCal object.
+ *
+ * Refreshes the calendar being accessed by the given backend.
+ *
+ * Since: 2.30
+ */
+void
+e_cal_backend_refresh (ECalBackend *backend, EDataCal *cal, EServerMethodContext context)
+{
+	g_return_if_fail (backend != NULL);
+	g_return_if_fail (E_IS_CAL_BACKEND (backend));
+
+	g_assert (CLASS (backend)->refresh != NULL);
+	(* CLASS (backend)->refresh) (backend, cal, context);
+}
+
 /**
  * e_cal_backend_remove:
  * @backend: A calendar backend.
@@ -658,7 +663,7 @@ e_cal_backend_remove (ECalBackend *backend, EDataCal *cal, EServerMethodContext
  *
  * Queries whether a calendar backend has been loaded yet.
  *
- * Return value: TRUE if the backend has been loaded with data, FALSE
+ * Returns: TRUE if the backend has been loaded with data, FALSE
  * otherwise.
  */
 gboolean
@@ -716,7 +721,7 @@ e_cal_backend_start_query (ECalBackend *backend, EDataCalView *query)
  *
  * Queries whether a calendar backend is connected remotely.
  *
- * Return value: The current mode the calendar is in
+ * Returns: The current mode the calendar is in
  **/
 CalMode
 e_cal_backend_get_mode (ECalBackend *backend)
@@ -908,7 +913,7 @@ e_cal_backend_create_object (ECalBackend *backend, EDataCal *cal, EServerMethodC
 	if (CLASS (backend)->create_object)
 		(* CLASS (backend)->create_object) (backend, cal, context, calobj);
 	else
-		e_data_cal_notify_object_created (cal, context, PermissionDenied, NULL, NULL);
+		e_data_cal_notify_object_created (cal, context, EDC_ERROR (UnsupportedMethod), NULL, NULL);
 }
 
 /**
@@ -930,7 +935,7 @@ e_cal_backend_modify_object (ECalBackend *backend, EDataCal *cal, EServerMethodC
 	if (CLASS (backend)->modify_object)
 		(* CLASS (backend)->modify_object) (backend, cal, context, calobj, mod);
 	else
-		e_data_cal_notify_object_removed (cal, context, PermissionDenied, NULL, NULL, NULL);
+		e_data_cal_notify_object_removed (cal, context, EDC_ERROR (UnsupportedMethod), NULL, NULL, NULL);
 }
 
 /**
@@ -1202,6 +1207,8 @@ match_query_and_notify (EDataCalView *query, const gchar *old_object, const gcha
  *
  * This methods has to be used before e_cal_backend_notify_view_progress.
  * Sets last notified percent value to 0.
+ *
+ * Since: 2.24
  **/
 void
 e_cal_backend_notify_view_progress_start (ECalBackend *backend)
@@ -1261,12 +1268,12 @@ e_cal_backend_notify_view_progress (ECalBackend *backend, const gchar *message,
 /**
  * e_cal_backend_notify_view_done:
  * @backend: A calendar backend.
- * @status: returns the status once the view is fully populated.
+ * @error: returns the error, if any, once the view is fully populated.
  *
  * Notifies each of the backend's listeners about the view_done in downloading the items.
  **/
 void
-e_cal_backend_notify_view_done (ECalBackend *backend, GNOME_Evolution_Calendar_CallStatus status)
+e_cal_backend_notify_view_done (ECalBackend *backend, const GError *error)
 {
 	ECalBackendPrivate *priv;
 	EList *queries;
@@ -1276,7 +1283,7 @@ e_cal_backend_notify_view_done (ECalBackend *backend, GNOME_Evolution_Calendar_C
 	priv = backend->priv;
 
 	if (priv->notification_proxy) {
-		e_cal_backend_notify_view_done (priv->notification_proxy, status);
+		e_cal_backend_notify_view_done (priv->notification_proxy, error);
 		return;
 	}
 
@@ -1288,7 +1295,7 @@ e_cal_backend_notify_view_done (ECalBackend *backend, GNOME_Evolution_Calendar_C
 
 		g_object_ref (query);
 
-		e_data_cal_view_notify_done (query, status);
+		e_data_cal_view_notify_done (query, error);
 
 		g_object_unref (query);
 
@@ -1394,18 +1401,33 @@ e_cal_backend_notify_object_removed (ECalBackend *backend, const ECalComponentId
 	g_object_unref (iter);
 }
 
+/**
+ * e_cal_backend_notify_objects_added:
+ *
+ * Since: 2.24
+ **/
 void
 e_cal_backend_notify_objects_added (ECalBackend *backend, EDataCalView *query, const GList *objects)
 {
 	e_data_cal_view_notify_objects_added (query, objects);
 }
 
+/**
+ * e_cal_backend_notify_objects_removed:
+ *
+ * Since: 2.24
+ **/
 void
 e_cal_backend_notify_objects_removed (ECalBackend *backend, EDataCalView *query, const GList *ids)
 {
 	e_data_cal_view_notify_objects_removed (query, ids);
 }
 
+/**
+ * e_cal_backend_notify_objects_modified:
+ *
+ * Since: 2.24
+ **/
 void
 e_cal_backend_notify_objects_modified (ECalBackend *backend, EDataCalView *query, const GList *objects)
 {
@@ -1484,6 +1506,8 @@ e_cal_backend_notify_error (ECalBackend *backend, const gchar *message)
  *
  * Empties backend's cache with all notifications and so on, thus all listening
  * will know there is nothing in this backend.
+ *
+ * Since: 2.28
  **/
 void
 e_cal_backend_empty_cache (ECalBackend *backend, ECalBackendCache *cache)
