@@ -26,18 +26,18 @@
 #endif
 
 #include <errno.h>
-#include <pthread.h>
 #include <stdio.h>
 
-#include <glib.h>
 #include <glib/gi18n-lib.h>
 
-#include "camel-exception.h"
 #include "camel-msgport.h"
 #include "camel-net-utils.h"
 #ifdef G_OS_WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
 #include "camel-net-utils-win32.h"
 #endif
+#include "camel-object.h"
 #include "camel-operation.h"
 
 #define d(x)
@@ -163,7 +163,6 @@ G_LOCK_DEFINE_STATIC (gethost_mutex);
 	}                                                              \
 } G_STMT_END
 
-
 #ifdef ENABLE_IPv6
 /* some helpful utils for IPv6 lookups */
 #define IPv6_BUFLEN_MIN  (sizeof (gchar *) * 3)
@@ -445,11 +444,14 @@ cs_freeinfo(struct _addrinfo_msg *msg)
 
 /* returns -1 if we didn't wait for reply from thread */
 static gint
-cs_waitinfo(gpointer (worker)(gpointer), struct _addrinfo_msg *msg, const gchar *error, CamelException *ex)
+cs_waitinfo (gpointer (worker)(gpointer),
+             struct _addrinfo_msg *msg,
+             const gchar *errmsg,
+             GError **error)
 {
 	CamelMsgPort *reply_port;
-	pthread_t id;
-	gint err, cancel_fd, cancel = 0, fd;
+	GThread *thread;
+	gint cancel_fd, cancel = 0, fd;
 
 	cancel_fd = camel_operation_cancel_fd(NULL);
 	if (cancel_fd == -1) {
@@ -459,7 +461,7 @@ cs_waitinfo(gpointer (worker)(gpointer), struct _addrinfo_msg *msg, const gchar
 
 	reply_port = msg->msg.reply_port = camel_msgport_new();
 	fd = camel_msgport_fd(msg->msg.reply_port);
-	if ((err = pthread_create(&id, NULL, worker, msg)) == 0) {
+	if ((thread = g_thread_create (worker, msg, TRUE, error)) != NULL) {
 		gint status;
 #ifndef G_OS_WIN32
 		GPollFD polls[2];
@@ -493,34 +495,39 @@ cs_waitinfo(gpointer (worker)(gpointer), struct _addrinfo_msg *msg, const gchar
 #endif
 						   ) {
 			if (status == -1)
-				camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM, "%s: %s", error,
+				g_set_error (
+					error, G_IO_ERROR,
+					g_io_error_from_errno (errno),
+					"%s: %s", errmsg,
 #ifndef G_OS_WIN32
-						     g_strerror(errno)
+					g_strerror (errno)
 #else
-						     g_win32_error_message (WSAGetLastError ())
+					g_win32_error_message (WSAGetLastError ())
 #endif
-						     );
+					);
 			else
-				camel_exception_setv(ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled"));
+				g_set_error (
+					error, G_IO_ERROR,
+					G_IO_ERROR_CANCELLED,
+					_("Cancelled"));
 
 			/* We cancel so if the thread impl is decent it causes immediate exit.
-			   We detach so we dont need to wait for it to exit if it isn't.
 			   We check the reply port incase we had a reply in the mean time, which we free later */
 			d(printf("Canceling lookup thread and leaving it\n"));
 			msg->cancelled = 1;
-			pthread_detach(id);
-			pthread_cancel(id);
+			g_thread_join (thread);
 			cancel = 1;
 		} else {
-			struct _addrinfo_msg *reply = (struct _addrinfo_msg *)camel_msgport_try_pop(reply_port);
+			struct _addrinfo_msg *reply;
 
-			g_assert(reply == msg);
 			d(printf("waiting for child to exit\n"));
-			pthread_join(id, NULL);
+			g_thread_join (thread);
 			d(printf("child done\n"));
+
+			reply = (struct _addrinfo_msg *)camel_msgport_try_pop(reply_port);
+			if (reply != msg)
+				g_warning ("%s: Received msg reply %p doesn't match msg %p", G_STRFUNC, reply, msg);
 		}
-	} else {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM, "%s: %s: %s", error, _("cannot create thread"), g_strerror(err));
 	}
 	camel_msgport_destroy(reply_port);
 
@@ -542,7 +549,8 @@ cs_getaddrinfo(gpointer data)
 	/* This is a pretty simplistic emulation of getaddrinfo */
 
 	while ((msg->result = camel_gethostbyname_r(msg->name, &h, msg->hostbufmem, msg->hostbuflen, &herr)) == ERANGE) {
-		pthread_testcancel();
+		if (msg->cancelled)
+			break;
                 msg->hostbuflen *= 2;
                 msg->hostbufmem = g_realloc(msg->hostbufmem, msg->hostbuflen);
 	}
@@ -599,7 +607,7 @@ cs_getaddrinfo(gpointer data)
 		}
 	}
 
-	for (i=0;h.h_addr_list[i];i++) {
+	for (i = 0; h.h_addr_list[i] && !msg->cancelled; i++) {
 		res = g_malloc0(sizeof(*res));
 		if (msg->hints) {
 			res->ai_flags = msg->hints->ai_flags;
@@ -629,9 +637,7 @@ cs_getaddrinfo(gpointer data)
 	}
 reply:
 	camel_msgport_reply((CamelMsg *)msg);
-	return NULL;
 cancel:
-	cs_freeinfo(msg);
 	return NULL;
 }
 #else
@@ -651,18 +657,23 @@ cs_getaddrinfo(gpointer data)
 			info->result = getaddrinfo(info->name, "443", info->hints, info->res);
 	}
 
-	if (info->cancelled) {
-		cs_freeinfo(info);
-	} else {
+	if (!info->cancelled)
 		camel_msgport_reply((CamelMsg *)info);
-	}
 
 	return NULL;
 }
 #endif /* NEED_ADDRINFO */
 
+/**
+ * camel_getaddrinfo:
+ *
+ * Since: 2.22
+ **/
 struct addrinfo *
-camel_getaddrinfo(const gchar *name, const gchar *service, const struct addrinfo *hints, CamelException *ex)
+camel_getaddrinfo (const gchar *name,
+                   const gchar *service,
+                   const struct addrinfo *hints,
+                   GError **error)
 {
 	struct _addrinfo_msg *msg;
 	struct addrinfo *res = NULL;
@@ -672,7 +683,10 @@ camel_getaddrinfo(const gchar *name, const gchar *service, const struct addrinfo
 	g_return_val_if_fail(name != NULL, NULL);
 
 	if (camel_operation_cancel_check(NULL)) {
-		camel_exception_set(ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled"));
+		g_set_error (
+			error, G_IO_ERROR,
+			G_IO_ERROR_CANCELLED,
+			_("Cancelled"));
 		return NULL;
 	}
 
@@ -698,21 +712,27 @@ camel_getaddrinfo(const gchar *name, const gchar *service, const struct addrinfo
 	msg->hostbuflen = 1024;
 	msg->hostbufmem = g_malloc(msg->hostbuflen);
 #endif
-	if (cs_waitinfo(cs_getaddrinfo, msg, _("Host lookup failed"), ex) == 0) {
+	if (cs_waitinfo(cs_getaddrinfo, msg, _("Host lookup failed"), error) == 0) {
 		if (msg->result != 0) {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, _("Host lookup failed: %s: %s"),
-					      name, gai_strerror (msg->result));
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Host lookup failed: %s: %s"),
+				name, gai_strerror (msg->result));
 		}
-
-		cs_freeinfo(msg);
 	} else
 		res = NULL;
 
+	cs_freeinfo (msg);
 	camel_operation_end(NULL);
 
 	return res;
 }
 
+/**
+ * camel_freeaddrinfo:
+ *
+ * Since: 2.22
+ **/
 void
 camel_freeaddrinfo(struct addrinfo *host)
 {
@@ -749,7 +769,8 @@ cs_getnameinfo(gpointer data)
 
 	while ((msg->result = camel_gethostbyaddr_r((const gchar *)&sin->sin_addr, sizeof(sin->sin_addr), AF_INET, &h,
 						    msg->hostbufmem, msg->hostbuflen, &herr)) == ERANGE) {
-		pthread_testcancel ();
+		if (msg->cancelled)
+			break;
                 msg->hostbuflen *= 2;
                 msg->hostbufmem = g_realloc(msg->hostbufmem, msg->hostbuflen);
 	}
@@ -773,10 +794,9 @@ cs_getnameinfo(gpointer data)
 	if (msg->serv)
 		sprintf(msg->serv, "%d", sin->sin_port);
 
-	camel_msgport_reply((CamelMsg *)msg);
-	return NULL;
+	if (!msg->cancelled)
+		camel_msgport_reply ((CamelMsg *)msg);
 cancel:
-	cs_freeinfo(msg);
 	return NULL;
 }
 #else
@@ -788,23 +808,34 @@ cs_getnameinfo(gpointer data)
 	/* there doens't appear to be a return code which says host or serv buffers are too short, lengthen them */
 	msg->result = getnameinfo(msg->addr, msg->addrlen, msg->host, msg->hostlen, msg->serv, msg->servlen, msg->flags);
 
-	if (msg->cancelled)
-		cs_freeinfo(msg);
-	else
+	if (!msg->cancelled)
 		camel_msgport_reply((CamelMsg *)msg);
 
 	return NULL;
 }
 #endif
 
+/**
+ * camel_getnameinfo:
+ *
+ * Since: 2.22
+ **/
 gint
-camel_getnameinfo(const struct sockaddr *sa, socklen_t salen, gchar **host, gchar **serv, gint flags, CamelException *ex)
+camel_getnameinfo (const struct sockaddr *sa,
+                   socklen_t salen,
+                   gchar **host,
+                   gchar **serv,
+                   gint flags,
+                   GError **error)
 {
 	struct _addrinfo_msg *msg;
 	gint result;
 
-	if (camel_operation_cancel_check(NULL)) {
-		camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled"));
+	if (camel_operation_cancel_check (NULL)) {
+		g_set_error (
+			error, G_IO_ERROR,
+			G_IO_ERROR_CANCELLED,
+			_("Cancelled"));
 		return -1;
 	}
 
@@ -828,11 +859,12 @@ camel_getnameinfo(const struct sockaddr *sa, socklen_t salen, gchar **host, gcha
 	msg->hostbuflen = 1024;
 	msg->hostbufmem = g_malloc(msg->hostbuflen);
 #endif
-	cs_waitinfo(cs_getnameinfo, msg, _("Name lookup failed"), ex);
+	cs_waitinfo(cs_getnameinfo, msg, _("Name lookup failed"), error);
 
 	if ((result = msg->result) != 0)
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, _("Name lookup failed: %s"),
-				      gai_strerror (result));
+		g_set_error (
+			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			_("Name lookup failed: %s"), gai_strerror (result));
 	else {
 		if (host)
 			*host = g_strdup(msg->host);
@@ -840,10 +872,7 @@ camel_getnameinfo(const struct sockaddr *sa, socklen_t salen, gchar **host, gcha
 			*serv = g_strdup(msg->serv);
 	}
 
-	g_free(msg->host);
-	g_free(msg->serv);
-	g_free(msg);
-
+	cs_freeinfo (msg);
 	camel_operation_end(NULL);
 
 	return result;
