@@ -54,6 +54,7 @@
 
 #include "camel-certdb.h"
 #include "camel-file-utils.h"
+#include "camel-net-utils.h"
 #include "camel-operation.h"
 #include "camel-session.h"
 #include "camel-stream-fs.h"
@@ -64,6 +65,8 @@
 #include <ws2tcpip.h>
 #endif
 
+#define d(x)
+
 #define IO_TIMEOUT (PR_TicksPerSecond() * 4 * 60)
 #define CONNECT_TIMEOUT (PR_TicksPerSecond () * 4 * 60)
 
@@ -179,11 +182,10 @@ tcp_stream_ssl_finalize (GObject *object)
 }
 
 static gssize
-tcp_stream_ssl_read (CamelStream *stream,
-                     gchar *buffer,
-                     gsize n)
+read_from_prfd (PRFileDesc *fd,
+                gchar *buffer,
+                gsize n)
 {
-	CamelTcpStreamSSL *tcp_stream_ssl = CAMEL_TCP_STREAM_SSL (stream);
 	PRFileDesc *cancel_fd;
 	gssize nread;
 
@@ -195,7 +197,7 @@ tcp_stream_ssl_read (CamelStream *stream,
 	cancel_fd = camel_operation_cancel_prfd (NULL);
 	if (cancel_fd == NULL) {
 		do {
-			nread = PR_Read (tcp_stream_ssl->priv->sockfd, buffer, n);
+			nread = PR_Read (fd, buffer, n);
 			if (nread == -1)
 				set_errno (PR_GetError ());
 		} while (nread == -1 && (PR_GetError () == PR_PENDING_INTERRUPT_ERROR ||
@@ -209,13 +211,13 @@ tcp_stream_ssl_read (CamelStream *stream,
 
 		/* get O_NONBLOCK options */
 		sockopts.option = PR_SockOpt_Nonblocking;
-		PR_GetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
+		PR_GetSocketOption (fd, &sockopts);
 		sockopts.option = PR_SockOpt_Nonblocking;
 		nonblock = sockopts.value.non_blocking;
 		sockopts.value.non_blocking = TRUE;
-		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
+		PR_SetSocketOption (fd, &sockopts);
 
-		pollfds[0].fd = tcp_stream_ssl->priv->sockfd;
+		pollfds[0].fd = fd;
 		pollfds[0].in_flags = PR_POLL_READ;
 		pollfds[1].fd = cancel_fd;
 		pollfds[1].in_flags = PR_POLL_READ;
@@ -242,7 +244,7 @@ tcp_stream_ssl_read (CamelStream *stream,
 				goto failed;
 			} else {
 				do {
-					nread = PR_Read (tcp_stream_ssl->priv->sockfd, buffer, n);
+					nread = PR_Read (fd, buffer, n);
 					if (nread == -1)
 						set_errno (PR_GetError ());
 				} while (nread == -1 && PR_GetError () == PR_PENDING_INTERRUPT_ERROR);
@@ -256,7 +258,7 @@ tcp_stream_ssl_read (CamelStream *stream,
 		error = errno;
 		sockopts.option = PR_SockOpt_Nonblocking;
 		sockopts.value.non_blocking = nonblock;
-		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
+		PR_SetSocketOption (fd, &sockopts);
 		errno = error;
 	}
 
@@ -264,11 +266,32 @@ tcp_stream_ssl_read (CamelStream *stream,
 }
 
 static gssize
-tcp_stream_ssl_write (CamelStream *stream,
-                      const gchar *buffer,
-                      gsize n)
+tcp_stream_ssl_read (CamelStream *stream,
+                     gchar *buffer,
+                     gsize n,
+                     GError **error)
+{
+	CamelTcpStreamSSL *ssl = CAMEL_TCP_STREAM_SSL (stream);
+	gssize result;
+
+	d (g_print ("SSL stream %p: reading %" G_GSIZE_FORMAT " bytes...\n", ssl, n));
+
+	result = read_from_prfd (ssl->priv->sockfd, buffer, n);
+
+	if (result == -1)
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
+			"%s", g_strerror (errno));
+
+	d (g_print ("SSL stream %p: read %" G_GSSIZE_FORMAT " bytes, errno = %d\n", ssl, result, result == -1 ? errno : 0));
+
+	return result;
+}
+
+static gssize
+write_to_prfd (PRFileDesc *fd, const gchar *buffer, gsize n)
 {
-	CamelTcpStreamSSL *tcp_stream_ssl = CAMEL_TCP_STREAM_SSL (stream);
 	gssize w, written = 0;
 	PRFileDesc *cancel_fd;
 
@@ -281,7 +304,7 @@ tcp_stream_ssl_write (CamelStream *stream,
 	if (cancel_fd == NULL) {
 		do {
 			do {
-				w = PR_Write (tcp_stream_ssl->priv->sockfd, buffer + written, n - written);
+				w = PR_Write (fd, buffer + written, n - written);
 				if (w == -1)
 					set_errno (PR_GetError ());
 			} while (w == -1 && (PR_GetError () == PR_PENDING_INTERRUPT_ERROR ||
@@ -299,13 +322,13 @@ tcp_stream_ssl_write (CamelStream *stream,
 
 		/* get O_NONBLOCK options */
 		sockopts.option = PR_SockOpt_Nonblocking;
-		PR_GetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
+		PR_GetSocketOption (fd, &sockopts);
 		sockopts.option = PR_SockOpt_Nonblocking;
 		nonblock = sockopts.value.non_blocking;
 		sockopts.value.non_blocking = TRUE;
-		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
+		PR_SetSocketOption (fd, &sockopts);
 
-		pollfds[0].fd = tcp_stream_ssl->priv->sockfd;
+		pollfds[0].fd = fd;
 		pollfds[0].in_flags = PR_POLL_WRITE;
 		pollfds[1].fd = cancel_fd;
 		pollfds[1].in_flags = PR_POLL_READ;
@@ -332,7 +355,7 @@ tcp_stream_ssl_write (CamelStream *stream,
 				errno = EINTR;
 			} else {
 				do {
-					w = PR_Write (tcp_stream_ssl->priv->sockfd, buffer + written, n - written);
+					w = PR_Write (fd, buffer + written, n - written);
 					if (w == -1)
 						set_errno (PR_GetError ());
 				} while (w == -1 && PR_GetError () == PR_PENDING_INTERRUPT_ERROR);
@@ -350,7 +373,7 @@ tcp_stream_ssl_write (CamelStream *stream,
 		error = errno;
 		sockopts.option = PR_SockOpt_Nonblocking;
 		sockopts.value.non_blocking = nonblock;
-		PR_SetSocketOption (tcp_stream_ssl->priv->sockfd, &sockopts);
+		PR_SetSocketOption (fd, &sockopts);
 		errno = error;
 	}
 
@@ -360,22 +383,56 @@ tcp_stream_ssl_write (CamelStream *stream,
 	return written;
 }
 
+static gssize
+tcp_stream_ssl_write (CamelStream *stream,
+                      const gchar *buffer,
+                      gsize n,
+                      GError **error)
+{
+	CamelTcpStreamSSL *ssl = CAMEL_TCP_STREAM_SSL (stream);
+	gssize result;
+
+	d (g_print ("SSL stream %p: writing %" G_GSIZE_FORMAT " bytes...\n", ssl, n));
+
+	result = write_to_prfd (ssl->priv->sockfd, buffer, n);
+
+	if (result == -1)
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
+			"%s", g_strerror (errno));
+
+	d (g_print ("SSL stream %p: wrote %" G_GSSIZE_FORMAT " bytes, errno = %d\n", ssl, result, result == -1 ? errno : 0));
+
+	return result;
+}
+
 static gint
-tcp_stream_ssl_flush (CamelStream *stream)
+tcp_stream_ssl_flush (CamelStream *stream,
+                      GError **error)
 {
 	/*return PR_Sync (((CamelTcpStreamSSL *)stream)->priv->sockfd);*/
 	return 0;
 }
 
 static gint
-tcp_stream_ssl_close (CamelStream *stream)
+tcp_stream_ssl_close (CamelStream *stream,
+                      GError **error)
 {
+	d (g_print ("SSL stream %p: closing\n", stream));
+
 	if (((CamelTcpStreamSSL *)stream)->priv->sockfd == NULL) {
 		errno = EINVAL;
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
+			"%s", g_strerror (errno));
 		return -1;
 	}
 
 	PR_Shutdown (((CamelTcpStreamSSL *)stream)->priv->sockfd, PR_SHUTDOWN_BOTH);
+
+	/* FIXME Need to set the GError if this fails. */
 	if (PR_Close (((CamelTcpStreamSSL *)stream)->priv->sockfd) == PR_FAILURE)
 		return -1;
 
@@ -652,15 +709,15 @@ camel_certdb_nss_cert_set(CamelCertDB *certdb, CamelCert *ccert, CERTCertificate
 	g_free (dir);
 
 	stream = camel_stream_fs_new_with_name (
-		path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+		path, O_WRONLY | O_CREAT | O_TRUNC, 0600, NULL);
 	if (stream != NULL) {
 		if (camel_stream_write (
 			stream, (const gchar *) ccert->rawcert->data,
-			ccert->rawcert->len) == -1) {
+			ccert->rawcert->len, NULL) == -1) {
 			g_warning ("Could not save cert: %s: %s", path, g_strerror (errno));
 			g_unlink (path);
 		}
-		camel_stream_close (stream);
+		camel_stream_close (stream, NULL);
 		g_object_unref (stream);
 	} else {
 		g_warning ("Could not save cert: %s: %s", path, g_strerror (errno));
@@ -937,6 +994,27 @@ enable_ssl (CamelTcpStreamSSL *ssl, PRFileDesc *fd)
 	return ssl_fd;
 }
 
+static PRFileDesc *
+enable_ssl_or_close_fd (CamelTcpStreamSSL *ssl, PRFileDesc *fd)
+{
+	PRFileDesc *ssl_fd;
+
+	ssl_fd = enable_ssl (ssl, fd);
+	if (ssl_fd == NULL) {
+		gint errnosave;
+
+		set_errno (PR_GetError ());
+		errnosave = errno;
+		PR_Shutdown (fd, PR_SHUTDOWN_BOTH);
+		PR_Close (fd);
+		errno = errnosave;
+
+		return NULL;
+	}
+
+	return ssl_fd;
+}
+
 static gint
 sockaddr_to_praddr(struct sockaddr *s, gint len, PRNetAddr *addr)
 {
@@ -977,9 +1055,8 @@ sockaddr_to_praddr(struct sockaddr *s, gint len, PRNetAddr *addr)
 	return -1;
 }
 
-static gint
-tcp_socket_ssl_connect (CamelTcpStream *stream,
-                        struct addrinfo *host)
+static PRFileDesc *
+tcp_socket_ssl_connect (CamelTcpStream *stream, struct addrinfo *host, gboolean possibly_use_ssl)
 {
 	CamelTcpStreamSSL *ssl = CAMEL_TCP_STREAM_SSL (stream);
 	PRNetAddr netaddr;
@@ -987,32 +1064,19 @@ tcp_socket_ssl_connect (CamelTcpStream *stream,
 
 	if (sockaddr_to_praddr(host->ai_addr, host->ai_addrlen, &netaddr) != 0) {
 		errno = EINVAL;
-		return -1;
+		return NULL;
 	}
 
 	fd = PR_OpenTCPSocket(netaddr.raw.family);
 	if (fd == NULL) {
 		set_errno (PR_GetError ());
-		return -1;
+		return NULL;
 	}
 
-	if (ssl->priv->ssl_mode) {
-		PRFileDesc *ssl_fd;
-
-		ssl_fd = enable_ssl (ssl, fd);
-		if (ssl_fd == NULL) {
-			gint errnosave;
-
-			set_errno (PR_GetError ());
-			errnosave = errno;
-			PR_Shutdown (fd, PR_SHUTDOWN_BOTH);
-			PR_Close (fd);
-			errno = errnosave;
-
-			return -1;
-		}
-
-		fd = ssl_fd;
+	if (possibly_use_ssl && ssl->priv->ssl_mode) {
+		fd = enable_ssl_or_close_fd (ssl, fd);
+		if (!fd)
+			return NULL;
 	}
 
 	cancel_fd = camel_operation_cancel_prfd(NULL);
@@ -1059,27 +1123,168 @@ tcp_socket_ssl_connect (CamelTcpStream *stream,
 			errnosave = errno;
 			PR_Shutdown (fd, PR_SHUTDOWN_BOTH);
 			PR_Close (fd);
-			ssl->priv->sockfd = NULL;
 			errno = errnosave;
 
-			return -1;
+			return NULL;
 		}
 
 		errno = 0;
 	}
 
-	ssl->priv->sockfd = fd;
+	return fd;
+}
 
-	return 0;
+static gboolean
+rehandshake_ssl (PRFileDesc *fd)
+{
+	if (SSL_ResetHandshake (fd, FALSE) == SECFailure) {
+		set_errno (PR_GetError ());
+		return FALSE;
+	}
+
+	if (SSL_ForceHandshake (fd) == SECFailure) {
+		set_errno (PR_GetError ());
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static PRFileDesc *
+connect_to_socks4_proxy (CamelTcpStreamSSL *ssl, const gchar *proxy_host, gint proxy_port, struct addrinfo *connect_addr)
+{
+	struct addrinfo *ai, hints;
+	gchar serv[16];
+	PRFileDesc *fd;
+	gchar request[9];
+	struct sockaddr_in *sin;
+	gchar reply[8];
+	gint save_errno;
+
+	g_assert (proxy_host != NULL);
+
+	d (g_print ("SSL stream %p: connecting to SOCKS4 proxy %s:%d {\n  resolving proxy host\n", ssl, proxy_host, proxy_port));
+
+	sprintf (serv, "%d", proxy_port);
+
+	memset (&hints, 0, sizeof (hints));
+	hints.ai_socktype = SOCK_STREAM;
+
+	ai = camel_getaddrinfo (proxy_host, serv, &hints, NULL);  /* NULL-GError */
+	if (!ai) {
+#ifdef G_OS_WIN32
+		errno = WSAEHOSTUNREACH;
+#else
+		errno = EHOSTUNREACH; /* FIXME: this is not an accurate error; we should translate the GError to an errno */
+#endif
+		d (g_print ("  camel_getaddrinfo() for the proxy failed\n}\n"));
+		return NULL;
+	}
+
+	d (g_print ("  creating socket and connecting\n"));
+
+	fd = tcp_socket_ssl_connect (CAMEL_TCP_STREAM (ssl), ai, FALSE);
+	save_errno = errno;
+
+	camel_freeaddrinfo (ai);
+
+	if (!fd) {
+		errno = save_errno;
+		d (g_print ("  could not connect: %d\n", errno));
+		goto error;
+	}
+
+	g_assert (connect_addr->ai_addr->sa_family == AF_INET); /* FIXME: what to do about IPv6?  Are we just screwed with SOCKS4? */
+	sin = (struct sockaddr_in *) connect_addr->ai_addr;
+
+	request[0] = 0x04;				/* SOCKS4 */
+	request[1] = 0x01;				/* CONNECT */
+	memcpy (request + 2, &sin->sin_port, 2);	/* port in network byte order */
+	memcpy (request + 4, &sin->sin_addr.s_addr, 4);	/* address in network byte order */
+	request[8] = 0x00;				/* terminator */
+
+	d (g_print ("  writing SOCKS4 request to connect to actual host\n"));
+	if (write_to_prfd (fd, request, sizeof (request)) != sizeof (request)) {
+		d (g_print ("  failed: %d\n", errno));
+		goto error;
+	}
+
+	d (g_print ("  reading SOCKS4 reply\n"));
+	if (read_from_prfd (fd, reply, sizeof (reply)) != sizeof (reply)) {
+		d (g_print ("  failed: %d\n", errno));
+		goto error;
+	}
+
+	if (!(reply[0] == 0		/* first byte of reply is 0 */
+	      && reply[1] == 90)) {	/* 90 means "request granted" */
+#ifdef G_OS_WIN32
+		errno = WSAECONNREFUSED;
+#else
+		errno = ECONNREFUSED;
+#endif
+		d (g_print ("  proxy replied with code %d\n", reply[1]));
+		goto error;
+	}
+
+	/* We are now proxied we are ready to send "normal" data through the socket */
+
+	if (ssl->priv->ssl_mode) {
+		d (g_print ("  enabling SSL\n"));
+		fd = enable_ssl_or_close_fd (ssl, fd);
+		if (!fd) {
+			d (g_print ("  could not enable SSL\n"));
+			goto error;
+		} else {
+			d (g_print ("  re-handshaking SSL\n"));
+			if (!rehandshake_ssl (fd)) {
+				d (g_print ("  failed\n"));
+				goto error;
+			}
+		}
+	}
+
+	d (g_print ("  success\n"));
+
+	goto out;
+
+error:
+	if (fd) {
+		save_errno = errno;
+		PR_Shutdown (fd, PR_SHUTDOWN_BOTH);
+		PR_Close (fd);
+		errno = save_errno;
+		fd = NULL;
+	}
+
+	d (g_print ("  returning errno %d\n", errno));
+
+out:
+
+	d (g_print ("}\n"));
+
+	return fd;
 }
 
 static gint
 tcp_stream_ssl_connect (CamelTcpStream *stream,
-                        struct addrinfo *host)
+                        struct addrinfo *host,
+                        GError **error)
 {
+	CamelTcpStreamSSL *ssl = CAMEL_TCP_STREAM_SSL (stream);
+	const gchar *proxy_host;
+	gint proxy_port;
+
+	camel_tcp_stream_peek_socks_proxy (stream, &proxy_host, &proxy_port);
+
 	while (host) {
-		if (tcp_socket_ssl_connect (stream, host) == 0)
+		if (proxy_host)
+			ssl->priv->sockfd = connect_to_socks4_proxy (ssl, proxy_host, proxy_port, host);
+		else
+			ssl->priv->sockfd = tcp_socket_ssl_connect (stream, host, TRUE);
+
+		if (ssl->priv->sockfd)
 			return 0;
+
 		host = host->ai_next;
 	}
 
@@ -1311,15 +1516,8 @@ camel_tcp_stream_ssl_enable_ssl (CamelTcpStreamSSL *ssl)
 
 		ssl->priv->sockfd = fd;
 
-		if (SSL_ResetHandshake (fd, FALSE) == SECFailure) {
-			set_errno (PR_GetError ());
+		if (!rehandshake_ssl (fd))
 			return -1;
-		}
-
-		if (SSL_ForceHandshake (fd) == SECFailure) {
-			set_errno (PR_GetError ());
-			return -1;
-		}
 	}
 
 	ssl->priv->ssl_mode = TRUE;
