@@ -347,6 +347,8 @@ e_source_combo_box_init (ESourceComboBox *source_combo_box)
  * from the provided #ESourceList.
  *
  * Returns: a new #ESourceComboBox
+ *
+ * Since: 2.22
  **/
 GtkWidget *
 e_source_combo_box_new (ESourceList *source_list)
@@ -366,6 +368,8 @@ e_source_combo_box_new (ESourceList *source_list)
  * @source_combo_box.
  *
  * Returns: an #ESourceList
+ *
+ * Since: 2.22
  **/
 ESourceList *
 e_source_combo_box_get_source_list (ESourceComboBox *source_combo_box)
@@ -382,6 +386,8 @@ e_source_combo_box_get_source_list (ESourceComboBox *source_combo_box)
  *
  * Sets the source list used by @source_combo_box to be @source_list.  This
  * causes the contents of @source_combo_box to be regenerated.
+ *
+ * Since: 2.22
  **/
 void
 e_source_combo_box_set_source_list (ESourceComboBox *source_combo_box,
@@ -427,6 +433,8 @@ e_source_combo_box_set_source_list (ESourceComboBox *source_combo_box,
  * if there is no active item.
  *
  * Returns: an #ESource or %NULL
+ *
+ * Since: 2.22
  **/
 ESource *
 e_source_combo_box_get_active (ESourceComboBox *source_combo_box)
@@ -455,6 +463,8 @@ e_source_combo_box_get_active (ESourceComboBox *source_combo_box)
  * @source: an #ESource
  *
  * Sets the active item to the one corresponding to @source.
+ *
+ * Since: 2.22
  **/
 void
 e_source_combo_box_set_active (ESourceComboBox *source_combo_box,
@@ -475,6 +485,8 @@ e_source_combo_box_set_active (ESourceComboBox *source_combo_box,
  * active item, or %NULL if there is no active item.
  *
  * Returns: a unique ID string or %NULL
+ *
+ * Since: 2.22
  **/
 const gchar *
 e_source_combo_box_get_active_uid (ESourceComboBox *source_combo_box)
@@ -496,8 +508,12 @@ e_source_combo_box_get_active_uid (ESourceComboBox *source_combo_box)
  * @uid: a unique ID of an #ESource
  *
  * Sets the active item to the one corresponding to @uid.
+ *
+ * Returns: whether found such @uid
+ *
+ * Since: 2.22
  **/
-void
+gboolean
 e_source_combo_box_set_active_uid (ESourceComboBox *source_combo_box,
                                    const gchar *uid)
 {
@@ -507,19 +523,24 @@ e_source_combo_box_set_active_uid (ESourceComboBox *source_combo_box,
 	GtkTreeIter iter;
 	gboolean iter_was_set;
 
-	g_return_if_fail (E_IS_SOURCE_COMBO_BOX (source_combo_box));
-	g_return_if_fail (uid != NULL);
+	g_return_val_if_fail (E_IS_SOURCE_COMBO_BOX (source_combo_box), FALSE);
+	g_return_val_if_fail (uid != NULL, FALSE);
 
 	priv = source_combo_box->priv;
 	combo_box = GTK_COMBO_BOX (source_combo_box);
 
 	reference = g_hash_table_lookup (priv->uid_index, uid);
-	g_return_if_fail (gtk_tree_row_reference_valid (reference));
+	if (!reference)
+		return FALSE;
+
+	g_return_val_if_fail (gtk_tree_row_reference_valid (reference), FALSE);
 
 	iter_was_set = gtk_tree_model_get_iter (
 		gtk_combo_box_get_model (combo_box), &iter,
 		gtk_tree_row_reference_get_path (reference));
-	g_return_if_fail (iter_was_set);
+	g_return_val_if_fail (iter_was_set, FALSE);
 
 	gtk_combo_box_set_active_iter (combo_box, &iter);
+
+	return TRUE;
 }
